#========================================================================================
# package      DE_EPAGES::ExternalShipping::t::AddressValidation
# state        public
#----------------------------------------------------------------------------------------
# description  Check Address Validation functionality
#========================================================================================
package DE_EPAGES::ExternalShipping::t::AddressValidation;

use strict;

use DE_EPAGES::ExternalShipping::API::AddressValidation;
use DE_EPAGES::Core::API::Script qw ( RunScript );

use DE_EPAGES::Test::Mock::EpagesObject;
use Test::More;

my %SHOP_ADDRESS_REQUIREMENTS = (
    'FirstName' => {
        'Mandatory' => 1,
        'Length' => { 'MinValue' => 1, 'MaxValue' => 50 },
    },
    'LastName'  => {
        'Mandatory' => 1,
        'Length' => { 'MinValue' => 1, 'MaxValue' => 50 },
    },
    'Street'    => {
        'Mandatory' => 1,
        'Length' => { 'MinValue' => 1, 'MaxValue' => 50 },
    },
    'Zipcode'   => {
        'Mandatory' => 1,
        'Length' => {
            'US'    => { 'MinValue' => 5, 'MaxValue' => 10 },
            'Other' => { 'MinValue' => 1, 'MaxValue' => 10 },
        },
    },
    'Country'   => {
        'Mandatory' => 1,
        'Enumeration' => ['US'],
    },
);

sub Main {
    testAddressValidationOnNoPersistence();
    done_testing();
}

#----------------------------------------------------------------------------------------
sub testAddressValidationOnNoPersistence {
    my $subTestName = ( caller (0) )[3];

    my $ShopAddress = _createValidShopAddress();
    my $AddressValidator = DE_EPAGES::ExternalShipping::API::AddressValidation->new($ShopAddress, \%SHOP_ADDRESS_REQUIREMENTS);
    $AddressValidator->validateAttributes();    # we have to call this to see the results in getStatesAsWarning
    my $aWarnings    = $AddressValidator->getStatesAsWarning();
    ok(scalar(@$aWarnings) == 0, "$subTestName -- Shop address is valid and no warnings were found");

    $ShopAddress = _createInvalidShopAddress();
    $AddressValidator = DE_EPAGES::ExternalShipping::API::AddressValidation->new($ShopAddress, \%SHOP_ADDRESS_REQUIREMENTS);
    $aWarnings    = $AddressValidator->getStatesAsWarning();
    my @WarningCodes = map {$_->{'Code'}} @{$aWarnings};
    is_deeply( \@WarningCodes,
               [ 'Spain is not an allowed country',
                 'FirstName is mandatory but not set',
                 'Zipcode is too Long',
                 'Street is mandatory but not set',
                 'LastName is too Long' ],
               "$subTestName -- Shop address is invalid and warnings were found");
}

#----------------------------------------------------------------------------------------
sub _createValidShopAddress {
    my $hAddressAttributes = {
        'AddressExtension' => 'RM 2100',
        'City'             => 'Washington',
        'Company'          => 'ePages',
        'Country'          => _createValidCountry(),
        'FirstName'         => 'John',
        'LastName'         => 'Doe',
        'State'            => 'DC',
        'Street'           => '475 L\'Enfant Plaza SW',
        'Zipcode'          => '20260',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Address::API::Object::Address', $hAddressAttributes );
}

#----------------------------------------------------------------------------------------
sub _createInvalidShopAddress {
    my $hAddressAttributes = {
        'AddressExtension' => 'RM 2100',
        'City'             => 'Washington',
        'Company'          => 'ePages',
        'Country'          => _createInvalidCountry(), # invalid (country not allow)
        'FirstName'         => '', # invalid length (minimum length is 1 character)
        'LastName'         => 'This is a longer than 50 characters last name so too long', # invalid
        'State'            => 'DC',
        'Street'           => undef, # invalid (mandatory field)
        'Zipcode'          => '12345-45678', # invalid zipcode length (maximum length is 10 characters)
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Address::API::Object::Address', $hAddressAttributes );
}

#----------------------------------------------------------------------------------------
sub _createValidCountry {
    my $hCountryAttributes = {
        'Code2'            => 'US',
        'Code3'            => 'USA',
        'CountryID'        => 840,
        'English'          => 'United States',
        'French'           => undef,
        'isEU'             => 0,
        'isUsed'           => 1,
        'Native'           => 'United States',
        'SubdivsMandatory' => 1,
    };
    return $hCountryAttributes;
}

#----------------------------------------------------------------------------------------
sub _createInvalidCountry {
    my $hCountryAttributes = {
        'Code2'            => 'ES',
        'Code3'            => 'ESP',
        'CountryID'        => 724,
        'English'          => 'Spain',
        'French'           => undef,
        'isEU'             => 1,
        'isUsed'           => 1,
        'Native'           => 'Espaa',
        'SubdivsMandatory' => 0,
    };
    return $hCountryAttributes;
}

RunScript( Sub => \&Main );

1;
