#========================================================================================
# package      DE_EPAGES::Order::t::SendShippingLabelAsPDFMail
# state        private
#----------------------------------------------------------------------------------------
# description  Having an issued shipping label with its PDF, it tests if we can send it
#               by email using the perl code which has been moved from USPS to Order's
#               cartridge (because it was almost the same).
#               It could/will be done also with other 3rd party shipping cartridges, as
#               SEUR, and so on... so all delivery cartridges will use the same code to
#               send their shipping labels as email attached PDFs.
#               This test checks the GenerateOrderDocumentPDF() in a case example where
#               (_IsShippingLabel($OrderDocument) && ExistsFile($pdfFilePath)) evals true;
#               otherwise this order's code is already checked by another test called
#               pdfIntegrationInvoice.t in this same folder.
#               If you want to see the PDF which is going to be attached to the email,
#               set this environment variable and run the script manually:
#                   [root@localvbox t]# export SEND_TESTMAIL_TO=youremail@epages.com
#                   [root@localvbox t]# $PERL SendShippingLabelAsPDFMail.t
#========================================================================================
package DE_EPAGES::Order::t::SendShippingLabelAsPDFMail;
use base DE_EPAGES::Shop::API::Test;

use strict;

use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::Object::API::Factory qw ( LoadObjectByPath ExistsObjectByPath LoadRootObject );
use DE_EPAGES::Core::API::File qw ( GetFileInfo CreateDirectory CopyFile DeleteFile );

use DE_EPAGES::Test::Mock::EpagesObject;
use Test::More;
use File::Basename qw ( dirname );

use DE_EPAGES::Order::API::Order qw ( GenerateOrderDocumentPDF );
use DE_EPAGES::MailType::API::TextMail qw ( SendTextMail );
use DE_EPAGES::Core::API::PerlTools qw ( isValid ) ;

use constant SHIPPING_LABEL_PDF => 'barcode_21496.pdf';
use constant PERSISTENT_DATA_DIRECTORY => dirname(__FILE__) . '/';
use constant ROOT_TEST_PATH => '/Test';


sub test {
    my $self = shift;

    $self->_testSendShippingLabelAsPDFMailOnNoPersistence();
    $self->_testSendShippingLabelAsPDFMailOnPersistence();

    done_testing();

    return;
}

#----------------------------------------------------------------------------------------

sub init {
    my $self = shift;

    return 1 if ExistsObjectByPath(ROOT_TEST_PATH) && $ENV{'SKIP_CLEANUP'};

    my $System = LoadRootObject();
    $System->insertChild( 'Object', { 'Alias' => 'Test' } );
    ImportObjects(PERSISTENT_DATA_DIRECTORY . 'import.xml', ROOT_TEST_PATH);

    $self->{'Shop'} = LoadObjectByPath( $self->shopPath );

    return 1;
}

#----------------------------------------------------------------------------------------

sub cleanup {
    my $self = shift;

    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath(ROOT_TEST_PATH)->delete if ExistsObjectByPath(ROOT_TEST_PATH);
    return 1;
}

#----------------------------------------------------------------------------------------

sub _testSendShippingLabelAsPDFMailOnNoPersistence {
    my $self = shift;

    my $SubTestName = ( caller (0) )[3];

    note( 'Test Validations On No Persistent data. Creating mock data...' );

    my $ShippingLabel = _createMockShippingLabel();

    my $ShippingLabelPDF = GenerateOrderDocumentPDF($ShippingLabel);

    is(
        $ShippingLabelPDF->{'filename'},
        'ShippingLabel_1018-1.pdf',
        $SubTestName.' - ShippingLabel_1018-1.pdf - Filename of the PDF which will be attached to the email, ok'
    );

    my $FileInfo = GetFileInfo(SHIPPING_LABEL_PDF);
    is(
        length($ShippingLabelPDF->{'pdf'}),
        $FileInfo->{'Size'},
        $SubTestName.' - '.$FileInfo->{'Size'}.' bytes - Size of the PDF which will be attached to the email, ok'
    );

    # only if wanted when script launched manually, create and send mail
    my $SEND_TESTMAIL_TO = $ENV{'SEND_TESTMAIL_TO'};
    if (isValid($SEND_TESTMAIL_TO)) {
        my %MailParams = (
            'from'        => 'nobody@epages.com',
            'to'          => $SEND_TESTMAIL_TO,
            'subject'     => 'testSendShippingLabelAsPDFMailOnNoPersistence',
            'content'     => 'Generated by SendShippingLabelAsPDFMail.t at '.$SubTestName,
            'attachments' => [{ 'filename' => $ShippingLabelPDF->{'filename'},
                                'data'     => $ShippingLabelPDF->{'pdf'},
                                'encoding' => 'base64' }],
        );
        SendTextMail(\%MailParams);
        note( 'PDF sent via email to '.$SEND_TESTMAIL_TO );
    }

    return undef;

}

#----------------------------------------------------------------------------------------

sub _createMockShippingLabel {

    # prepare attributes
    my $hShippingLabelAttributes = {
        'instanceOf'   => 'ShippingLabel',
        'ID'           => '21496',
        'Alias'        => '1018-1',
        'PrivatePath'  => '../../Order/t',
        'Order'       => _createMockOrder(),
        'Class'        => _createShippingLabelClass(),
    };
    # mock object with params
    my $ShippingLabel = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::ExternalShipping::API::Object::ShippingLabel',
        $hShippingLabelAttributes
    );
    # mock object functions
    $ShippingLabel->mock(
        'class',
        sub {
            return $hShippingLabelAttributes->{'Class'};
        }
    );
    return $ShippingLabel;

}

#----------------------------------------------------------------------------------------

sub _createMockOrder {

    my $hOrderAttributes = {
        'LineItemContainer' => _createMockLineItemContainer(),
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::Order::API::Object::CustomerOrder',
        $hOrderAttributes
    );

}

#----------------------------------------------------------------------------------------

sub _createShippingLabelClass {

    my $hShippingLabelClass = {
        'Alias' => 'ShippingLabel',
        'NameOrAlias' => 'ShippingLabel',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new(
        '',
        $hShippingLabelClass
    );

}

#----------------------------------------------------------------------------------------

sub _createMockLineItemContainer {

    my $hLineItemContainerAttributes = {
        'LanguageID' => '2',
        'LocaleID' => 'en_GB',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::Order::API::Object::LineItemContainer',
        $hLineItemContainerAttributes
    );

}

#----------------------------------------------------------------------------------------

sub _testSendShippingLabelAsPDFMailOnPersistence {
    my $self = shift;

    my $SubTestName = ( caller (0) )[3];

    note( 'Test Validations On Persistent data. Importing test data...' );

    my $Shop = $self->shop;
    my $ShippingLabel = $Shop->path('TestCustomer/Orders/1234-5678/ShippingLabels/1234-5678-1');
    my $ShippingLabelID = $ShippingLabel->get('ID');
    my $ShippingLabelPrivatePath = $ShippingLabel->get('PrivatePath');

    CreateDirectory( $ShippingLabelPrivatePath, {'recursive' => 1} );
    my $CopyTarget = $ShippingLabelPrivatePath.'/barcode_'.$ShippingLabelID.'.pdf';
    CopyFile (SHIPPING_LABEL_PDF, $CopyTarget);
    note( $SubTestName.' - PDF copied and renamed - '.$CopyTarget );

    my $ShippingLabelPDF = GenerateOrderDocumentPDF($ShippingLabel);

    is(
        $ShippingLabelPDF->{'filename'},
        'ShippingLabel_1234-5678-1.pdf',
        $SubTestName.' - ShippingLabel_1234-5678-1.pdf - Filename of the PDF which will be attached to the email, ok'
    );

    my $FileInfo = GetFileInfo(SHIPPING_LABEL_PDF);
    is(
        length($ShippingLabelPDF->{'pdf'}),
        $FileInfo->{'Size'},
        $SubTestName.' - '.$FileInfo->{'Size'}.' bytes - Size of the PDF which will be attached to the email, ok'
    );

    # only if wanted when script launched manually, create and send mail
    my $SEND_TESTMAIL_TO = $ENV{'SEND_TESTMAIL_TO'};
    if (isValid($SEND_TESTMAIL_TO)) {
        my %MailParams = (
            'from'        => 'nobody@epages.com',
            'to'          => $SEND_TESTMAIL_TO,
            'subject'     => 'testSendShippingLabelAsPDFMailOnPersistence',
            'content'     => 'Generated by SendShippingLabelAsPDFMail.t at '.$SubTestName,
            'attachments' => [{ 'filename' => $ShippingLabelPDF->{'filename'},
                                'data'     => $ShippingLabelPDF->{'pdf'},
                                'encoding' => 'base64' }],
        );
        SendTextMail(\%MailParams);
        note( 'PDF sent via email to '.$SEND_TESTMAIL_TO );
    }

    is(
        1,
        DeleteFile($CopyTarget),
        $SubTestName.' - TempFile '.$CopyTarget.' deleted ok'
    );

    return undef;

}

#=========================================================================================

__PACKAGE__->runScript;
1;
