use strict;
use Test::More tests => 38;

use DE_EPAGES::Core::API::Error qw( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Database::API::Connection qw( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw( LoadRootObject );
use DE_EPAGES::WebService::API::Client;

sub Main {
    my $ShopAlias = 'WSShop';
    my $StoreName = 'Store';
    my $ShopTypeAlias = 'ECom1000';
    my $ShopRefAlias = $StoreName.'.'.$ShopAlias;
    my $DomainName = 'meinshop.de';
    my $aSecondaryDomains = [qw(test.shop.de testshop.de)];

    my $User     = '/Providers/Distributor/Users/admin';
    my $Password = 'admin';
    # create a web service client

    my $WebserviceURL;
    RunOnStore( Store => GetTestSite(), Sub => sub {
        my $h = LoadRootObject()->get([qw(DomainName WebServerDirectory WebServerScriptNamePart)]);
        $WebserviceURL = "http://$h->{DomainName}/$h->{WebServerDirectory}/$h->{WebServerScriptNamePart}.soap";
    });

    my $ShopConfigService = DE_EPAGES::WebService::API::Client
        ->uri('urn://epages.de/WebService/ShopConfigService/2014/03')
        ->proxy($WebserviceURL, timeout => 900 );
    $ShopConfigService->userinfo( "$User:$Password" );

    ok( 1, 'ShopConfigService' );

    # delete in case of previous errors
    $ShopConfigService->deleteShopRef({
        Alias => $ShopRefAlias
    });
    $ShopConfigService->deleteShopRef({
        Alias => $ShopRefAlias.'2'
    });

    ok( 1, 'deleteShop before' );

    # create a shop
    my $ShopID = $ShopConfigService->create({
        Database => $StoreName,
        ShopType => $ShopTypeAlias,
        ShopAlias => $ShopAlias,
        Alias => $ShopRefAlias,
        DomainName => $DomainName,
        SecondaryDomains => $aSecondaryDomains,
    })->result;
print "ShopID=$ShopID\n";
    ok( $ShopID > 1, 'createShop' );

    # check defaults
    my $hInfo = $ShopConfigService->getInfo({ Alias => $ShopRefAlias })->result;
    ok( $ShopTypeAlias eq $hInfo->{ShopType}, 'getInfo' );
    ok( !$hInfo->{IsDeleted}, 'default: IsDeleted=0' );
    ok( !$hInfo->{IsTrialShop}, 'default: IsTrialShop=0' );
    ok( !$hInfo->{IsInternalTestShop}, 'default: IsInternalTestShop=0' );
    ok( !$hInfo->{HasSSLCertificate}, 'default: HasSSLCertificate=0' );
    ok( !$hInfo->{IsMarkedForDel}, 'default: IsMarkedForDel=0' );
    is( $hInfo->{WebServerScriptNamePart}, undef, 'default: WebServerScriptNamePart=undef' );
    is( $hInfo->{MerchantLogin}, 'admin', 'default: MerchantLogin=admin' );
    is( $hInfo->{DomainName}, $DomainName, 'getInfo: DomainName' );
    is_deeply( $hInfo->{SecondaryDomains}, $aSecondaryDomains, 'getInfo: SecondaryDomains' );

    # get all shop infos
    my $ahAllInfo = $ShopConfigService->getAllInfo->result;
    ok( scalar @$ahAllInfo >= 1, 'at least one shop' );
    my ($hTestShopInfo) = grep { $_->{Alias} eq $ShopRefAlias } @$ahAllInfo;
    ok( defined $hTestShopInfo, 'test shop found in allInfo' );
    ok( $hTestShopInfo->{Database} eq $StoreName, 'allInfo[i].StoreName' );
    ok( $hTestShopInfo->{ShopType} eq $ShopTypeAlias, 'allInfo[i].ShopType' );
    ok( $hInfo->{DomainName} eq $DomainName, 'allInfo[i].DomainName' );
    is_deeply( $hInfo->{SecondaryDomains}, $aSecondaryDomains, 'allInfo[i].SecondaryDomains' );


    # create with invalid domain name
    eval {
        $ShopConfigService->create({
            Database => $StoreName,
            ShopType => $ShopTypeAlias,
            ShopAlias => $ShopAlias . '2',
            Alias => $ShopRefAlias . '2',
            DomainName => '###',
        });
    };
    ok( ExistsError() && GetError()->vars->{'FaultString'} eq 'The domain name ### is not valid',
        'update invalid domain name' );

    # create with duplicate domain name
    eval {
        $ShopConfigService->create({
            Database => $StoreName,
            ShopType => $ShopTypeAlias,
            ShopAlias => $ShopAlias . '2',
            Alias => $ShopRefAlias . '2',
            DomainName => $DomainName,
        });
    };
    ok( ExistsError() && GetError()->vars->{'FaultString'} eq "The domain name $DomainName is already used by another site",
        'create with duplicate domain name' );

    my $aUpdatedDomains = ['www.deinshop.com', 'www.seinshop.com'];
    # update the shop
    $ShopConfigService->update({
        Alias => $ShopRefAlias,
        ShopType => 'ECom5000',
        DomainName => 'www.meinshop.com',
        SecondaryDomains => $aUpdatedDomains,
        IsTrialShop => 1,
        IsInternalTestShop => 1,
    });

    $hInfo = $ShopConfigService->getInfo({ Alias => $ShopRefAlias })->result;
    ok( $hInfo->{ShopType} eq 'ECom5000', 'update ShopType' );
    ok( $hInfo->{DomainName} eq 'www.meinshop.com', 'update DomainName' );
    is_deeply( $hInfo->{'SecondaryDomains'}, $aUpdatedDomains, 'update SecondaryDomains' );
    ok( $hInfo->{IsTrialShop}, 'update IsTrialShop' );
    ok( $hInfo->{IsInternalTestShop}, 'update IsInternalTestShop' );

    # update with invalid domain name
    eval {
        $ShopConfigService->update({ Alias => $ShopRefAlias, DomainName => '###' });
    };
    ok( ExistsError() && GetError()->vars->{'FaultString'} eq 'The domain name ### is not valid',
        'update invalid domain name' );

    # update with IDN-Domain
    my $IDNDomain = "www.m\x{00FC}ller.de";
    $ShopConfigService->update({ Alias => $ShopRefAlias, DomainName => $IDNDomain });

    $hInfo = $ShopConfigService->getInfo({ Alias => $ShopRefAlias })->result;
    ok( $hInfo->{DomainName} eq $IDNDomain, 'Update IDN Domain' );

    # setSecondaryDomains
    my $aNewSecondaryDomains = [qw(test.shop2.de testshop2.de)];
    $ShopConfigService->setSecondaryDomains({ Alias => $ShopRefAlias }, $aNewSecondaryDomains);
    $hInfo = $ShopConfigService->getInfo({ Alias => $ShopRefAlias })->result;
    is_deeply( $hInfo->{SecondaryDomains}, $aNewSecondaryDomains, 'setSecondaryDomains' );

    # update the shop and mark as deleted
    $ShopConfigService->update({
        Alias => $ShopRefAlias,
        MarkedForDelete => 1,
    });

    my $IsMarkedForDeletion = $ShopConfigService->getInfo({ Alias => $ShopRefAlias })->result->{IsMarkedForDel};
    my $IsClosed = $ShopConfigService->getInfo({ Alias => $ShopRefAlias })->result->{IsClosed};
    ok( $IsClosed, 'By marking the shop as deleted the flag IsClosed has been set too.' );

    # Reset the MarkedForDelete-flag.
    $ShopConfigService->update({
        Alias => $ShopRefAlias,
        MarkedForDelete => 0,
    });

    $IsMarkedForDeletion = $ShopConfigService->getInfo({ Alias => $ShopRefAlias })->result->{IsMarkedForDel};
    ok( !$IsMarkedForDeletion, 'Resetting the flag MarkedForDelete is possible.' );

    # Set IsClosed to false and test if setting the MarkedForDelete-flag got reset too.
    $ShopConfigService->update({
        Alias => $ShopRefAlias,
        MarkedForDelete => 1,
    });
    $ShopConfigService->update({
        Alias => $ShopRefAlias,
        IsClosed => 0,
    });

    $IsMarkedForDeletion = $ShopConfigService->getInfo({ Alias => $ShopRefAlias })->result->{IsMarkedForDel};
    ok( !$IsMarkedForDeletion, 'MarkedForDelete has been reset after re-opening the shop.' );

    # create with duplicate IDN domain name
    eval {
        $ShopConfigService->create({
            Database => $StoreName,
            ShopType => $ShopTypeAlias,
            ShopAlias => $ShopAlias . '2',
            Alias => $ShopRefAlias . '2',
            DomainName => $IDNDomain,
        });
    };
    ok( ExistsError() && GetError()->vars->{'FaultString'} eq 'The domain name www.xn--mller-kva.de is already used by another site',
        'create with duplicate IDN domain name' );

    # delete the shop
    $ShopConfigService->delete({
        Alias => $ShopRefAlias
    });

    my $IsDeleted = $ShopConfigService->getInfo({ Alias => $ShopRefAlias })->result->{IsDeleted};
    ok( $IsDeleted, 'flag IsDeleted is set by delete()' );

    my $Exists = $ShopConfigService->exists({ Alias => $ShopRefAlias })->result;
    ok( $Exists, 'shop still exist before deleteShopRef()' );

    # delete ShopRef
    $ShopConfigService->deleteShopRef({
        Alias => $ShopRefAlias
    });

    $Exists = $ShopConfigService->exists({ Alias => $ShopRefAlias })->result;
    ok( !$Exists, 'shop does not exist after deleteShopRef()' );

    # create a shop in the default database
    $ShopID = $ShopConfigService->create({
        ShopType => $ShopTypeAlias,
        ShopAlias => $ShopAlias,
        Alias => $ShopRefAlias,
    });
    $hInfo = $ShopConfigService->getInfo({ Alias => $ShopRefAlias })->result;

    ok( $ShopID, "createShop in default database: $hInfo->{Database}" );

    # delete ShopRef (also deletes the shop data in the store database)
    $ShopConfigService->deleteShopRef({
        Alias => $ShopRefAlias
    });
    $ShopConfigService->deleteShopRef({
        Alias => $ShopRefAlias.'2'
    });

    $Exists = $ShopConfigService->exists({ Alias => $ShopRefAlias })->result;
    ok( !$Exists, 'shop does not exist after deleteShopRef()' );
}

RunScript( Sub => \&Main );
