#========================================================================================
# package      DE_EPAGES::USPS::t::AddressInformation_external
# state        public
#----------------------------------------------------------------------------------------
# description  external integration tests for USPS Address Information API
#               (Verify USPS tag)
#========================================================================================
package DE_EPAGES::USPS::t::AddressInformation_external;

use strict;

use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::USPS::API::WebServices::AddressInformation;
use DE_EPAGES::USPS::t::PermanenceTestingBuilder;
use Test::More;
use Test::Exception;

sub Main {

    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            if ( $ENV{SKIP_EXTERNAL_TESTS} ) {
                plan skip_all => 'call USPS Server (SKIP_EXTERNAL_TESTS == 1)';
            }

            eval {
                TestRequestValidationOnNoPersistence();
                TestRequestValidationOnPersistence();
            };
            if ( ExistsError() ) {
                my $Error = GetError();
                fail( 'Error: ' . $Error->longMessage );
            }

            done_testing();
        }
    );
}

#----------------------------------------------------------------------------------------
sub TestRequestValidationOnNoPersistence {
    my $subTestName = ( caller (0) )[3];

    my $ShippingFulfillmentMethod = _createShippingFulfillmentMethod();
    my $AddressInfo = DE_EPAGES::USPS::API::WebServices::AddressInformation->new($ShippingFulfillmentMethod);

    #now feed AddressInformattion object and check validity
    _feedAddressInfoObject($AddressInfo);
    ok( $AddressInfo->isValidRequestOrError(),
        "$subTestName - called isValidRequestOrError validator on a filled request" );

    $AddressInfo->callWebService();
    ok( $AddressInfo->isValidResponseOrError(), "$subTestName - called isValidResponseOrError on a valid Response" );

    my @ResponseAddressesArray = $AddressInfo->getAddressesOrEmpty();
    is( scalar (@ResponseAddressesArray), 1, "$subTestName - called getAddressesOrEmpty and Response has 1 Addresses" );

    my $Address         = $ResponseAddressesArray[0];
    my $ExpectedAddress = _createExpectedAddress();
    is_deeply( $Address, $ExpectedAddress,
        "$subTestName - called getAddressesOrEmpty and check that the structure is as expected" );

    my $Address2 = $Address->{'Address2'};
    is( $Address2, '6406 IVY LN',
        "$subTestName - called getAddressesOrEmpty and check that the Address2 is as expected" );

    #let's do tests with wrong credentials
    $AddressInfo->setCredentials('WRONGUSERNAME');
    $AddressInfo->callWebService();
    ok( $AddressInfo->isErrorResponse(), "$subTestName - called isErrorResponse by using invalid credentials" );

    #let's do test with a wrong address
    my $AddressInfoWithWrongRequest
      = DE_EPAGES::USPS::API::WebServices::AddressInformation->new($ShippingFulfillmentMethod);
    _feedAddressInfoObjectWithWrongData($AddressInfoWithWrongRequest);
    $AddressInfoWithWrongRequest->callWebService();
    throws_ok(
        sub { $AddressInfoWithWrongRequest->isValidResponseOrError() },
        qr/INVALID_ADDRESS/,
        "$subTestName - called isValidResponseOrError validator on an invalid address (City and Zip Code not matched)"
    );

    return;
}

#----------------------------------------------------------------------------------------
sub TestRequestValidationOnPersistence {
    my $subTestName = ( caller (0) )[3];

    my $TestBuilderUSPS    = DE_EPAGES::USPS::t::PermanenceTestingBuilder->new('TestShopPermanentDataForTestingUSPS.xml');


  SKIP:
    {
        ok( $TestBuilderUSPS->isShopImported(), 'Test Shop for USPS testing imported into DataBase' ) or
            skip('Persistent Tests for USPS skipped');

        my $ShippingFulfillmentMethod = $TestBuilderUSPS->getShippingFulfillmentMethod();
        my $ShippingLabel =
          $TestBuilderUSPS->getShopDependentObject('Customers/TestCustomer/Orders/5002/ShippingLabels/5002-1');

        my $AddressInformation1
          = DE_EPAGES::USPS::API::WebServices::AddressInformation->new($ShippingFulfillmentMethod);
        $AddressInformation1->setCredentials( $ShippingFulfillmentMethod->get('WebserviceUsername') );
        my $Shop = $TestBuilderUSPS->getShop();
        $AddressInformation1->setAddress( $Shop->get('Address') );

        ok( $AddressInformation1->callWebService(),
            "$subTestName - called callWebService method on a filled request sourced from DB (Shop Address)" );

        my $AddressInformation2
          = DE_EPAGES::USPS::API::WebServices::AddressInformation->new($ShippingFulfillmentMethod);
        $AddressInformation2->setCredentials( $ShippingFulfillmentMethod->get('WebserviceUsername') );
        $AddressInformation2->setAddress( $ShippingLabel->get('Address') );

        ok( $AddressInformation2->callWebService(),
            "$subTestName - called callWebService method on a filled request sourced from DB (5002-1 shipping label address)"
        );

        $TestBuilderUSPS->cleanupTestEnvironment();
    }

    return;
}

#----------------------------------------------------------------------------------------
sub _feedAddressInfoObject {
    my ($AddressInfo) = @_;

    my $ShippingFulfillmentMethod = _createShippingFulfillmentMethod();
    $AddressInfo->setCredentials( $ShippingFulfillmentMethod->get('WebserviceUsername') );

    my $Address = _createAddress();
    $AddressInfo->setAddress($Address);
    return;
}

#----------------------------------------------------------------------------------------
sub _feedAddressInfoObjectWithWrongData {
    my ($AddressInfo) = @_;

    my $ShippingFulfillmentMethod = _createShippingFulfillmentMethod();
    $AddressInfo->setCredentials( $ShippingFulfillmentMethod->get('WebserviceUsername') );

    my $Address = _createWrongAddress();
    $AddressInfo->setAddress($Address);
    return;
}

#----------------------------------------------------------------------------------------
sub _createShippingFulfillmentMethod {
    my $hShippingFulfillmentMethodAttributes = {
        'WebserviceUsername'      => '605EPAGE1719',
        'Site'                    => _createMockShop(),
        'ShippingFulfillmentType' => _createShippingFulfillmentType(),
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::USPS::API::Object::ShippingFulfillmentMethodUSPS',
        $hShippingFulfillmentMethodAttributes );

}

#----------------------------------------------------------------------------------------
sub _createMockShop {
    my $hShopAttributes = { 'Alias' => 'Shop', 'IsInternalTestShop' => 1 };
    my $Shop = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Shop::API::Object::Shop', $hShopAttributes );
    return $Shop;
}

#----------------------------------------------------------------------------------------
sub _createShippingFulfillmentType {
    my $hShippingFulfillmentTypeAttributes = {
        'WebserviceURL'     => 'https://stg-secure.shippingapis.com',
        'WebserviceTestURL' => 'https://stg-secure.shippingapis.com'
    };
    my $ShippingFulfillmentType
      = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::ExternalShipping::API::Object::ShippingFulfillmentType',
        $hShippingFulfillmentTypeAttributes );
    return $ShippingFulfillmentType;
}

#----------------------------------------------------------------------------------------
sub _createAddress {
    my $hAddressAttributes = {
        'AddressExtension' => '',
        'City'             => 'GREENBELT',
        'Company'          => '',
        'State'            => 'MD',
        'Street'           => '6406 IVY LN',
        'Zipcode'          => '20770-1441',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Address::API::Object::Address', $hAddressAttributes );
}

#----------------------------------------------------------------------------------------
sub _createWrongAddress {
    my $hAddressAttributes = {
        'AddressExtension' => '',
        'City'             => 'NONAMECITY',
        'Company'          => '',
        'State'            => 'MA',
        'Street'           => 'PHANTOMSTREET',
        'Zipcode'          => '12345',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Address::API::Object::Address', $hAddressAttributes );
}

#----------------------------------------------------------------------------------------
sub _createExpectedAddress {

    my $ExpectedAddress = {
        'ID'       => '0',
        'Address2' => '6406 IVY LN',
        'City'     => 'GREENBELT',
        'State'    => 'MD',
        'Zip5'     => '20770',
        'Zip4'     => '1441',
    };

    return $ExpectedAddress;
}

#=========================================================================================

RunScript( Sub => \&Main );

1;
