#========================================================================================
# package      DE_EPAGES::USPS::t::AddressInformation_internal
# state        public
#----------------------------------------------------------------------------------------
# description  internal integration tests for USPS AddressInformation API
#               (Verify USPS tag)
#========================================================================================
package DE_EPAGES::USPS::t::AddressInformation_internal;

use strict;

use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );

use DE_EPAGES::USPS::API::WebServices::AddressInformation;
use DE_EPAGES::USPS::t::MockedCalls qw ( RemoteClientMocker );
use DE_EPAGES::USPS::t::PermanenceTestingBuilder;
use DE_EPAGES::Test::Mock::EpagesObject;
use Test::Exception;
use Test::More;
use File::Basename qw ( dirname );

use constant TEST_RESPONSES_DIRECTORY => dirname(__FILE__) . '/TestResponses/';

#----------------------------------------------------------------------------------------
sub Main {

    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            eval {
                TestRequestValidationOnNoPersistence();
                TestRequestValidationOnPersistence();
            };
            if ( ExistsError() ) {
                my $Error = GetError();
                fail( 'Error: ' . $Error->longMessage );
            }

            done_testing();

        }
    );
}

#----------------------------------------------------------------------------------------
sub TestRequestValidationOnNoPersistence {
    my $subTestName               = ( caller (0) )[3];
    my $ShippingFulfillmentMethod = _createShippingFulfillmentMethod();
    my $AddressInfo = DE_EPAGES::USPS::API::WebServices::AddressInformation->new($ShippingFulfillmentMethod);

    throws_ok(
        sub { $AddressInfo->isValidRequestOrError() },
        qr/INVALID_REQUEST_NOT_VALID_NODE_ATTRIBUTES/,
        "$subTestName - called isValidRequestOrError validator on a not filled request (no credentials)"
    );

    $AddressInfo->setCredentials( $ShippingFulfillmentMethod->get('WebserviceUsername') );
    throws_ok( sub { $AddressInfo->isValidRequestOrError() },
        qr/STRING_EMPTY/,
        "$subTestName - called isValidRequestOrError validator on a not filled request (mandatory strings empty)" );

    _feedAddressInfoObjectWithWrongAddress($AddressInfo);
    throws_ok(
        sub { $AddressInfo->isValidRequestOrError() },
        qr/Address2 is empty/,
        "$subTestName - Request without Address2"
    );

    _feedAddressInfoObject($AddressInfo);
    ok( $AddressInfo->isValidRequestOrError(),
        "$subTestName - called isValidRequestOrError validator on a filled request" );

    #check Positive responses
    my $Client
      = RemoteClientMocker( TEST_RESPONSES_DIRECTORY . 'AddressInformation_positive_multiple_response.xml', '200' );
    my $AddressInfoWithPositiveMultipleResponse
      = DE_EPAGES::USPS::API::WebServices::AddressInformation->new( $ShippingFulfillmentMethod, $Client );
    _feedAddressInfoObject($AddressInfoWithPositiveMultipleResponse);
    $AddressInfoWithPositiveMultipleResponse->callWebService();
    ok(
        $AddressInfoWithPositiveMultipleResponse->isValidResponseOrError(),
        "$subTestName - called isValidResponseOrError on a valid error"
    );

    my @ResponseAddressesArray = $AddressInfoWithPositiveMultipleResponse->getAddressesOrEmpty();
    is( scalar (@ResponseAddressesArray), 5, "$subTestName - called getAddressesOrEmpty and Response has 5 Addresses" );

    my $Address         = $ResponseAddressesArray[0];
    my $ExpectedAddress = _createExpectedAddress();
    is_deeply( $Address, $ExpectedAddress,
        "$subTestName - called getAddressesOrEmpty and check that the structure is as expected" );

    $Client = RemoteClientMocker( TEST_RESPONSES_DIRECTORY . 'AddressInformation_positive_response.xml', '200' );
    my $AddressInfoWithPositiveResponse
      = DE_EPAGES::USPS::API::WebServices::AddressInformation->new( $ShippingFulfillmentMethod, $Client );
    _feedAddressInfoObject($AddressInfoWithPositiveResponse);
    $AddressInfoWithPositiveResponse->callWebService();
    ok(
        $AddressInfoWithPositiveResponse->isValidResponseOrError(),
        "$subTestName - called isValidResponseOrError on a valid error"
    );

    my $Firm = $ResponseAddressesArray[0]->{'FirmName'};
    is( $Firm, 'XYZ Corp.', "$subTestName - called getAddressesOrEmpty and FirmName is XYZ Corp." );

    #check Negative responses
    $Client = RemoteClientMocker( TEST_RESPONSES_DIRECTORY . 'AddressInformation_negative_response.xml', '200' );

    my $AddressInfoWithErrorResponse
      = DE_EPAGES::USPS::API::WebServices::AddressInformation->new( $ShippingFulfillmentMethod, $Client );
    _feedAddressInfoObject($AddressInfoWithErrorResponse);
    $AddressInfoWithErrorResponse->callWebService();
    ok( $AddressInfoWithErrorResponse->isErrorResponse(),
        "$subTestName - called isErrorResponse validator on an erroneous response" );

    like( $AddressInfoWithErrorResponse->getErrorResponseDescriptionOrEmpty(),
        qr/^Invalid/, "$subTestName - called getErrorResponseDescriptionOrEmpty" );

    is( $AddressInfoWithErrorResponse->getErrorResponseNumberOrEmpty(),
        '-2147218002', "$subTestName - called getErrorResponseNumberOrEmpty" );

    is(
        $AddressInfoWithErrorResponse->getErrorResponseSourceOrEmpty(),
        'SOLServerPickupTest;SOLServerPickupTest.CarrierPickupAvailability_Respond',
        "$subTestName - called getErrorResponseSourceOrEmpty"
    );

    is( $AddressInfoWithErrorResponse->getErrorResponseHelpFileOrEmpty(),
        '', "$subTestName - called getErrorResponseHelpFileOrEmpty" );

    is( $AddressInfoWithErrorResponse->getErrorResponseHelpContextOrEmpty(),
        '', "$subTestName - called getErrorHelpContextOrEmpty" );

    $Client = RemoteClientMocker( TEST_RESPONSES_DIRECTORY . 'Failed_Connection.txt', '500' );
    my $AddressInfoWithErrorConnection
      = DE_EPAGES::USPS::API::WebServices::AddressInformation->new( $ShippingFulfillmentMethod, $Client );
    _feedAddressInfoObject($AddressInfoWithErrorConnection);
    throws_ok( sub { $AddressInfoWithErrorConnection->callWebService() },
        qr/CONNECTION_FAILED/, "$subTestName - catched a Connection Failed error" );

    $Client = RemoteClientMocker( TEST_RESPONSES_DIRECTORY . 'AddressInformation_address_error_response.xml', '200' );
    my $AddressInfoWithAddressErrorResponse
      = DE_EPAGES::USPS::API::WebServices::AddressInformation->new( $ShippingFulfillmentMethod, $Client );
    _feedAddressInfoObjectThatGeneratesAddressErrorResponse($AddressInfoWithAddressErrorResponse);
    $AddressInfoWithAddressErrorResponse->callWebService();
    throws_ok( sub { $AddressInfoWithAddressErrorResponse->isValidResponseOrError() },
        qr/INVALID_ADDRESS/, "$subTestName - catched an Address Error" );

    return;
}

#----------------------------------------------------------------------------------------
sub TestRequestValidationOnPersistence {
    my $subTestName = ( caller (0) )[3];

    my $TestBuilderUSPS    = DE_EPAGES::USPS::t::PermanenceTestingBuilder->new('TestShopPermanentDataForTestingUSPS.xml');

  SKIP:
    {
        ok( $TestBuilderUSPS->isShopImported(), 'Test Shop for USPS testing imported into DataBase' ) or
            skip('Persistent Tests for USPS skipped');

        my $ShippingFulfillmentMethod = $TestBuilderUSPS->getShippingFulfillmentMethod();
        my $ShippingLabel
          = $TestBuilderUSPS->getShopDependentObject('Customers/TestCustomer/Orders/5002/ShippingLabels/5002-1');
        my $Client = RemoteClientMocker( TEST_RESPONSES_DIRECTORY . 'AddressInformation_positive_response.xml', '200' );
        my $AddressInformation1
          = DE_EPAGES::USPS::API::WebServices::AddressInformation->new( $ShippingFulfillmentMethod, $Client );
        my $Shop = $TestBuilderUSPS->getShop();
        $AddressInformation1->setCredentials( $ShippingFulfillmentMethod->get('WebserviceUsername') );
        $AddressInformation1->setAddress( $Shop->get('Address') );

        ok( $AddressInformation1->callWebService(),
            "$subTestName - called callWebService method on a filled request sourced from DB (Shop Address)" );

        my $AddressInformation2
          = DE_EPAGES::USPS::API::WebServices::AddressInformation->new( $ShippingFulfillmentMethod, $Client );
        $AddressInformation2->setCredentials( $ShippingFulfillmentMethod->get('WebserviceUsername') );
        $AddressInformation2->setAddress( $ShippingLabel->get('Address') );

        ok( $AddressInformation2->callWebService(),
            "$subTestName - called callWebService method on a filled request sourced from DB (5002-1 shipping label address)"
        );

        $TestBuilderUSPS->cleanupTestEnvironment();
    }

    return;
}

#----------------------------------------------------------------------------------------
sub _feedAddressInfoObject {
    my ($AddressInfo) = @_;

    my $ShippingFulfillmentMethod = _createShippingFulfillmentMethod();
    $AddressInfo->setCredentials( $ShippingFulfillmentMethod->get('WebserviceUsername') );

    my $Address = _createAddress();
    $AddressInfo->setAddress($Address);
    return;
}

#----------------------------------------------------------------------------------------
sub _createShippingFulfillmentMethod {
    my $hShippingFulfillmentMethodAttributes = {
        'WebserviceUsername'      => '605EPAGE1719',
        'Site'                    => _createMockShop(),
        'ShippingFulfillmentType' => _createShippingFulfillmentType(),
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::USPS::API::Object::ShippingFulfillmentMethodUSPS',
        $hShippingFulfillmentMethodAttributes );

}

#----------------------------------------------------------------------------------------
sub _createMockShop {
    my $hShopAttributes = { 'Alias' => 'Shop', 'IsInternalTestShop' => 1 };
    my $Shop = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Shop::API::Object::Shop', $hShopAttributes );
    return $Shop;
}

#----------------------------------------------------------------------------------------
sub _createShippingFulfillmentType {
    my $hShippingFulfillmentTypeAttributes = {
        'WebserviceURL'     => 'https://stg-secure.shippingapis.com',
        'WebserviceTestURL' => 'https://stg-secure.shippingapis.com'
    };
    my $ShippingFulfillmentType
      = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::ExternalShipping::API::Object::ShippingFulfillmentType',
        $hShippingFulfillmentTypeAttributes );
    return $ShippingFulfillmentType;
}

#----------------------------------------------------------------------------------------
sub _createAddress {
    my $hAddressAttributes = {
        'AddressExtension' => '',
        'City'             => 'GREENBELT',
        'Company'          => 'XYZ Corp.',
        'State'            => 'MD',
        'Urbanization'     => '',
        'Street'           => '6406 IVY LN',
        'Zipcode'          => '20770-1441',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Address::API::Object::Address', $hAddressAttributes );
}

#----------------------------------------------------------------------------------------
sub _createExpectedAddress {

    my $ExpectedAddress = {
        'ID'       => '0',
        'FirmName' => 'XYZ Corp.',
        'Address1' => 'Apartment 1',
        'Address2' => '6406 IVY LN',
        'City'     => 'GREENBELT',
        'State'    => 'MD',
        'Zip5'     => '20770',
        'Zip4'     => '1441',
    };

    return $ExpectedAddress;
}

#----------------------------------------------------------------------------------------
sub _feedAddressInfoObjectWithWrongAddress {
    my ($AddressInfo) = @_;

    my $ShippingFulfillmentMethod = _createShippingFulfillmentMethod();
    $AddressInfo->setCredentials( $ShippingFulfillmentMethod->get('WebserviceUsername') );

    my $Address = _createWrongAddress();
    $AddressInfo->setAddress($Address);
    return;
}

#----------------------------------------------------------------------------------------
sub _createWrongAddress {
    my $hAddressAttributes = {
        'AddressExtension' => '',
        'City'             => 'GREENBELT',
        'Company'          => 'XYZ Corp.',
        'State'            => 'MD',
        'Urbanization'     => '',
        'Street'           => '',
        'Zipcode'          => '20770-1441',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Address::API::Object::Address', $hAddressAttributes );
}

#----------------------------------------------------------------------------------------
sub _feedAddressInfoObjectThatGeneratesAddressErrorResponse {
    my ($AddressInfo) = @_;

    my $ShippingFulfillmentMethod = _createShippingFulfillmentMethod();
    $AddressInfo->setCredentials( $ShippingFulfillmentMethod->get('WebserviceUsername') );

    my $Address = _createAddressThatGeneratesAddressErrorResponse();
    $AddressInfo->setAddress($Address);
    return;
}

#----------------------------------------------------------------------------------------
sub _createAddressThatGeneratesAddressErrorResponse {
    my $hAddressAttributes = {
        'AddressExtension' => '',
        'City'             => 'NotMatchWithZipCode',
        'Company'          => 'XYZ Corp.',
        'State'            => 'MD',
        'Urbanization'     => '',
        'Street'           => 'Street 4',
        'Zipcode'          => '12345',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Address::API::Object::Address', $hAddressAttributes );
}


#=========================================================================================

RunScript( Sub => \&Main );

1;
