#========================================================================================
# package      DE_EPAGES::USPS::t::JSON
# state        public
#----------------------------------------------------------------------------------------
# description  tests for JSON calls
#========================================================================================
package DE_EPAGES::USPS::t::JSON;

use strict;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError Error);
use DE_EPAGES::Core::API::PerlTools qw ( GetLocalTimeZone );
use Test::Exception;
use Test::More;
use DE_EPAGES::Test::Mock::EpagesObject;
use HTTP::Request;
use HTTP::Response;
use HTTP::Status qw( HTTP_OK );
use DE_EPAGES::Presentation::Servlet::JSON;
use DE_EPAGES::USPS::UI::JSON;

sub Main {
    plan tests => 15;
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            eval { USPSJSONCalls(); };
            if ( ExistsError() ) {
                my $Error = GetError();
                fail( 'Error: ' . $Error->longMessage() );
            }
            done_testing();
        }
    );
}
#----------------------------------------------------------------------------------------
sub USPSJSONCalls {
    my $SubTestName = ( caller (0) )[3];
    my $Servlet = _createServletObject();

    my $MockedPositiveJSON = _mockPositiveJSONObject();
    my $MockedNegativeJSON = _mockNegativeJSONObject();

    #Tests Positive Availability Request
    my $hReturnPositiveAvailability = $MockedPositiveJSON->JSONUSPSPickUpAvailabilityRequest($Servlet);
    is( $hReturnPositiveAvailability->{'ReturnCode'},
        '0', "$SubTestName - Check returned return code Value for Availability positive response. It should be 0" );
    is( $hReturnPositiveAvailability->{'DayOftheWeek'},
        'Wednesday', "$SubTestName - Check returned Day Of the Week Value for Availability positive response. It should be Wednesday" );
    is( $hReturnPositiveAvailability->{'CarrierRoute'},
        'C', "$SubTestName - Check returned Carrier Route Value for Availability positive response. It should be C" );
    is( $hReturnPositiveAvailability->{'ReturnedDate'},
        '09-11-2014', "$SubTestName - Check Returned Date for Availability positive response. It should be 09-11-2014" );

    #Tests Positive Schedule Request
    my $hReturnPositiveSchedule = $MockedPositiveJSON->JSONUSPSPickUpRequest($Servlet);
    is( $hReturnPositiveSchedule->{'ReturnCode'},
        '0', "$SubTestName - Check returned return code Value for Schedule positive response. It should be 0" );
    is( $hReturnPositiveSchedule->{'ReturnMessage'},
        'WTC12345434', "$SubTestName - Check returned Confirmation Number Value for Schedule positive response. It should be WTC12345434" );
    is( $hReturnPositiveSchedule->{'ScheduledDay'},
        '05/15/2014', "$SubTestName - Check returned Date for Schedule positive response. It should be 05/15/2014" );

    #Tests Positive Cancel Request
    my $hReturnPositiveCancel = $MockedPositiveJSON->JSONUSPSPickUpCancelRequest($Servlet);
    is( $hReturnPositiveCancel->{'ReturnCode'},
        '0', "$SubTestName - Check returned return code Value for Cancel positive response. It should be 0" );
    is( $hReturnPositiveCancel->{'ReturnMessage'},
        'Cancel', "$SubTestName - Check returned Cancelation message for Cancel positive response. It should be Cancel" );

    #Tests Negative Availability Request
    my $hReturnNegativeAvailability = $MockedNegativeJSON->JSONUSPSPickUpAvailabilityRequest($Servlet);
    is( $hReturnNegativeAvailability->{'ReturnCode'},
        '1', "$SubTestName - Check returned return code Value for Availability negative response. It should be 1" );
    is( $hReturnNegativeAvailability->{'ErrorDesc'},
        'Error in Availability', "$SubTestName - Check returned Error message for Availability. It should be Error in Availability" );

    #Tests Negative Schedule Request
    my $hReturnNegativeSchedule = $MockedNegativeJSON->JSONUSPSPickUpRequest($Servlet);
    is( $hReturnNegativeSchedule->{'ReturnCode'},
        '1', "$SubTestName - Check returned return code Value for Schedule negative response. It should be 1" );
    is( $hReturnNegativeSchedule->{'ErrorDesc'},
        'Error in Schedule', "$SubTestName - Check returned Error message for Schedule. It should be Error in Schedule" );

    #Tests Negative Cancel Request
    my $hReturnNegativeCancel = $MockedNegativeJSON->JSONUSPSPickUpCancelRequest($Servlet);
    is( $hReturnNegativeCancel->{'ReturnCode'},
        '1', "$SubTestName - Check returned return code Value for Cancel negative response. It should be 1" );
    is( $hReturnNegativeCancel->{'ErrorDesc'},
        'Error in Cancel', "$SubTestName - Check returned Error message for Cancel. It should be Error in Cancel" );
}
#----------------------------------------------------------------------------------------
sub _createServletObject {
    my $ServletObject = DE_EPAGES::Presentation::Servlet::JSON->new();
    $ServletObject->{'Request'}  = HTTP::Request->new( GET => 'http://www.example.com/' );
    $ServletObject->{'Response'} = HTTP::Response->new( HTTP_OK );

    my $MockedServletObject = Test::MockObject::Extends->new( $ServletObject );
    $MockedServletObject->mock('form',sub{
                my $self = shift;
                my $Form = DE_EPAGES::Presentation::API::Form->new( $ServletObject->request() );
                my $date = DateTime->today( time_zone => GetLocalTimeZone() );
                $Form->value('PickUpDate', $date);
                $Form->value('PackageLocation', 'Mail Room');
                $Form->value('SpecialInstructions', 'abcd');
                return $Form;
        });
    $MockedServletObject->mock('object',sub{
                return _createShippingLabel();
        });
    return $MockedServletObject;
}
#----------------------------------------------------------------------------------------
sub _createShippingLabel {
    my $hLabelAttributes = {
        'ImageType'                  => 'None',
        'ParcelDepth'                => '5.30',
        'ParcelLength'               => '5.20',
        'ParcelWidth'                => '10.30',
        'ParcelGirth'                => '8.70',
        'PickupDate'                 => DateTime->today( time_zone => GetLocalTimeZone() ),
        'ShippingFulfillmentProduct' => _createShippingFulfillmentProduct(),
        'ShippingFulfillmentMethod'  => _createShippingFulfillmentMethod(),
        'SizeUnit'                   => _createInchUnitOfMeasurement(),
        'TotalWeight'                => '10',
        'WeightUnit'                 => _createOunceUnitOfMeasurement(),
        'ServiceType'                => _createServiceType(),
        'PackageLocation'            => _createPackageLocation(),
        'SpecialInstructions'        => 'ring the bell',
        'Site'                       => _createMockShop(),
        'ConfirmationNumber'         => '',
        'IsPickUpRequested'          => '',
    };

    my $ShippingLabel = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::USPS::API::Object::ShippingLabelUSPS',
        $hLabelAttributes );

    return $ShippingLabel;
}
#----------------------------------------------------------------------------------------
sub _createMockShop {
    my $hShopAttributes = { 'Alias' => 'Shop', 'Address' => _createAddress() , 'LocaleID' => 'en_US'};
    my $Shop = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::Shop::API::Object::Shop',
        $hShopAttributes );
    $Shop->mock('loadObject',sub{return;});
    return $Shop;
}
#----------------------------------------------------------------------------------------
sub _MockCommonMethods{
    my ($PackagePickupObject) = @_;

    $PackagePickupObject->mock('callWebService',sub{return;});
    $PackagePickupObject->mock('setAddress',sub{return;});
    $PackagePickupObject->mock('setCredentials',sub{return;});
    $PackagePickupObject->mock('feedPackagePickupAvailabilityObject',sub{return;});
    $PackagePickupObject->mock('feedPackagePickupScheduleObject',sub{return;});
    $PackagePickupObject->mock('feedPackagePickupCancelObject',sub{return;});
    $PackagePickupObject->mock('feedRequest',sub{return;});

    return $PackagePickupObject;

}
#----------------------------------------------------------------------------------------
sub _createPackagePickUpCancelObject{
    my $hPackagePickupCancelAttributes = { 'Alias' => 'PickupCancel'};
    my $PackagePickupCancel = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::USPS::API::WebServices::PackagePickupCancel',
        $hPackagePickupCancelAttributes );
    return $PackagePickupCancel;
}
#----------------------------------------------------------------------------------------
sub _createPackagePickUpAvailabilityObject{
    my $hPackagePickupAvailabilityAttributes = { 'Alias' => 'PickupAvailability'};
    my $PackagePickupAvailability = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::USPS::API::WebServices::PackagePickupAvailability',
        $hPackagePickupAvailabilityAttributes );

    return $PackagePickupAvailability;
}
#----------------------------------------------------------------------------------------
sub _createPackagePickUpScheduleObject{
    my $hPackagePickupScheduleAttributes = { 'Alias' => 'PickupSchedule'};
    my $PackagePickupSchedule = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::USPS::API::WebServices::PackagePickupSchedule',
        $hPackagePickupScheduleAttributes );

    return $PackagePickupSchedule;
}
#----------------------------------------------------------------------------------------
sub _createPositiveMockPickupCancel {

    my $PackagePickupCancel = _createPackagePickUpCancelObject();
    $PackagePickupCancel = _MockCommonMethods($PackagePickupCancel);
    $PackagePickupCancel->mock('isValidResponseOrError',sub{return;});
    $PackagePickupCancel->mock('setConfirmationNumber',sub{return;});
    $PackagePickupCancel->mock('getStatusOrEmpty',sub{return 'Cancel';});
    return $PackagePickupCancel;
}
#----------------------------------------------------------------------------------------
sub _createPositiveMockPickupAvailability {
    my $PackagePickupAvailability = _createPackagePickUpAvailabilityObject();
    $PackagePickupAvailability = _MockCommonMethods($PackagePickupAvailability);
    $PackagePickupAvailability->mock('setPickUpDateOrError',sub{return;});
    $PackagePickupAvailability->mock('isValidResponseOrError',sub{return;});
    $PackagePickupAvailability->mock('getDayOfWeekOrEmpty',sub{return 'Wednesday';});
    $PackagePickupAvailability->mock('getDateOrEmpty',sub{return '09-11-2014';});
    $PackagePickupAvailability->mock('getCarrierRouteOrEmpty',sub{return 'C';});

    return $PackagePickupAvailability;
}
#----------------------------------------------------------------------------------------
sub _createPositiveMockPickupSchedule {

    my $PackagePickupSchedule = _createPackagePickUpScheduleObject();
    $PackagePickupSchedule = _MockCommonMethods($PackagePickupSchedule);
    $PackagePickupSchedule->mock('isValidResponseOrError',sub{return;});
    $PackagePickupSchedule->mock('setPickupSettings',sub{return;});
    $PackagePickupSchedule->mock('getDateOrEmpty',sub{return '05/15/2014';});
    $PackagePickupSchedule->mock('getConfirmationNumberOrEmpty',sub{return 'WTC12345434';});

    return $PackagePickupSchedule;
}
#----------------------------------------------------------------------------------------
sub _createNegativeMockPickupCancel {

    my $PackagePickupCancel = _createPackagePickUpCancelObject();
    $PackagePickupCancel = _MockCommonMethods($PackagePickupCancel);
    $PackagePickupCancel->mock('setConfirmationNumber',sub{return;});
    $PackagePickupCancel->mock('isValidResponseOrError',sub{Error('Error in Cancel');});
    $PackagePickupCancel->mock('getStatusOrEmpty',sub{return;});
    return $PackagePickupCancel;
}
#----------------------------------------------------------------------------------------
sub _createNegativeMockPickupAvailability {
    my $PackagePickupAvailability = _createPackagePickUpAvailabilityObject();
    $PackagePickupAvailability = _MockCommonMethods($PackagePickupAvailability);
    $PackagePickupAvailability->mock('setPickUpDateOrError',sub{return;});
    $PackagePickupAvailability->mock('isValidResponseOrError',sub{Error('Error in Availability');});
    $PackagePickupAvailability->mock('getDayOfWeekOrEmpty',sub{return;});
    $PackagePickupAvailability->mock('getDateOrEmpty',sub{return;});
    $PackagePickupAvailability->mock('getCarrierRouteOrEmpty',sub{return;});

    return $PackagePickupAvailability;
}
#----------------------------------------------------------------------------------------
sub _createNegativeMockPickupSchedule {

    my $PackagePickupSchedule = _createPackagePickUpScheduleObject();
    $PackagePickupSchedule = _MockCommonMethods($PackagePickupSchedule);
    $PackagePickupSchedule->mock('setPickupSettings',sub{return;});
    $PackagePickupSchedule->mock('isValidResponseOrError',sub{Error('Error in Schedule');});
    $PackagePickupSchedule->mock('getConfirmationNumberOrEmpty',sub{return;});

    return $PackagePickupSchedule;
}
#----------------------------------------------------------------------------------------
sub _createJSONObject{
    my $JSON = DE_EPAGES::USPS::UI::JSON->new();
    my $MockedJSON = Test::MockObject::Extends->new( $JSON );
    $MockedJSON->mock('writeContentOutput',sub{
                my $self = shift;
                my ($rContent) = @_;
                return $rContent;
    });

    return $MockedJSON;
}
#----------------------------------------------------------------------------------------
sub _mockNegativeJSONObject{
    my $MockedNegativeJSON = _createJSONObject();

    $MockedNegativeJSON->mock('_createPackagePickupCancel',sub{
                my $self = shift;
                return _createNegativeMockPickupCancel();
    });
    $MockedNegativeJSON->mock('_createPackagePickupAvailability',sub{
                my $self = shift;
                return _createNegativeMockPickupAvailability();
    });
    $MockedNegativeJSON->mock('_createPackagePickupSchedule',sub{
                my $self = shift;
                return _createNegativeMockPickupSchedule();
    });

    return $MockedNegativeJSON;
}
#----------------------------------------------------------------------------------------
sub _mockPositiveJSONObject{
    my $MockedPositiveJSON = _createJSONObject();

    $MockedPositiveJSON->mock('_createPackagePickupCancel',sub{
                my $self = shift;
                return _createPositiveMockPickupCancel();
    });
    $MockedPositiveJSON->mock('_createPackagePickupAvailability',sub{
                my $self = shift;
                return _createPositiveMockPickupAvailability();
    });
    $MockedPositiveJSON->mock('_createPackagePickupSchedule',sub{
                my $self = shift;
                return _createPositiveMockPickupSchedule();
    });

    return $MockedPositiveJSON;
}
#----------------------------------------------------------------------------------------
sub _createAddress {
    my $hAddressAttributes = {
        'AddressExtension' => 'RM 2100',
        'City'             => 'Washington',
        'Company'          => 'ePages',
        'FirstName'        => 'John',
        'LastName'         => 'Doe',
        'Phone'            => '123456789',
        'Fullname'         => 'John Doe',
        'State'            => 'DC',
        'Street'           => '475 Enfant Plaza SW',
        'Zipcode'          => '20260',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Address::API::Object::Address', $hAddressAttributes );
}
#----------------------------------------------------------------------------------------
sub _createShippingFulfillmentProduct {
    my $hProductAttributes = { 'Alias' => 'PriorityMail', };
    return DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::ExternalShipping::API::Object::ShippingFulfillmentProduct',
        $hProductAttributes );
}
#----------------------------------------------------------------------------------------
sub _createServiceType {
    my $hServiceTypeAttributes = { 'Alias' => 'PRIORITY_MAIL', 'USPSWebservice2ndKey' => 'PriorityMail'};
    return DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::USPS::API::Object::ListItemUSPS',
        $hServiceTypeAttributes );
}
#----------------------------------------------------------------------------------------
sub _createPackageLocation {
    my $hPackageLocationAttributes = { 'Alias' => 'BACK_DOOR ', 'USPSWebserviceKey' => 'Back door'};
    return DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::USPS::API::Object::ListItemUSPS',
        $hPackageLocationAttributes );
}
#----------------------------------------------------------------------------------------
sub _createShippingFulfillmentMethod {
    my $hShippingFulfillmentMethod = { 'WebserviceUsername' => '605EPAGE1719' };
    return DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::ExternalShipping::API::Object::ShippingFulfillmentMethod',
        $hShippingFulfillmentMethod );
}
#----------------------------------------------------------------------------------------
sub _createInchUnitOfMeasurement {
    my $hMetreAttributes = {
        'Alias' => 'metre',
        'Path'  => '/Units/metre',
    };
    my $metre = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Object::API::Object::UnitOfMeasurement',
        $hMetreAttributes );
    my $hInchAttributes = {
        'Alias'  => 'inch',
        'Factor' => 0.0254,
        'Parent' => $metre,
        'Path'   => '/Units/metre/inch',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Object::API::Object::UnitOfMeasurement',
        $hInchAttributes );
}
#----------------------------------------------------------------------------------------
sub _createOunceUnitOfMeasurement {
    my $hGramAttributes = { 'Alias' => 'gram' };
    my $Gram = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Object::API::Object::UnitOfMeasurement',
        $hGramAttributes );
    my $hOunceAttributes = {
        'Alias'  => 'ounce',
        'Factor' => 28.3495,
        'Parent' => $Gram,
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Object::API::Object::UnitOfMeasurement',
        $hOunceAttributes );
}

RunScript( Sub => \&Main );

1;