#========================================================================================
# package      DE_EPAGES::USPS::t::PackagePickupAvailability_external
# state        public
#----------------------------------------------------------------------------------------
# description  external integration tests for USPS package Pickup Availability API
#               (CarrierPickupAvilability USPS tag)
#========================================================================================
package DE_EPAGES::USPS::t::PackagePickupAvailability_external;

use strict;

use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );

use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::USPS::API::WebServices::PackagePickupAvailability;
use DE_EPAGES::USPS::t::PermanenceTestingBuilder;
use Test::More;
use Test::Exception;

sub Main {

    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            if ( $ENV{SKIP_EXTERNAL_TESTS} ) {
                plan skip_all => 'call USPS Server (SKIP_EXTERNAL_TESTS == 1)';
            }

            eval {
                TestRequestValidationOnNoPersistence();
                TestRequestValidationOnPersistence();
            };
            if ( ExistsError() ) {
                my $Error = GetError();
                fail( 'Error: ' . $Error->longMessage );
            }

            done_testing();

        }
    );
}

#----------------------------------------------------------------------------------------
sub TestRequestValidationOnNoPersistence {
    my $subTestName = ( caller (0) )[3];

    my $ShippingFulfillmentMethod = _createShippingFulfillmentMethod();
    my $PackagePickupAvailability
      = DE_EPAGES::USPS::API::WebServices::PackagePickupAvailability->new($ShippingFulfillmentMethod);

    #now feed TrackingDeliveryConfirmationLabel object and check validity
    _feedAddressInfoObject($PackagePickupAvailability);
    ok( $PackagePickupAvailability->isValidRequestOrError(),
        "$subTestName - called isValidRequestOrError validator on a filled request" );

    $PackagePickupAvailability->callWebService();

    #lauch response and check response is valid

    ok( $PackagePickupAvailability->isValidResponseOrError(),
        "$subTestName - called isValidResponseOrError validator on a response" );
}

#----------------------------------------------------------------------------------------
sub TestRequestValidationOnPersistence {
    my $subTestName = ( caller (0) )[3];

    my $TestBuilderUSPS    = DE_EPAGES::USPS::t::PermanenceTestingBuilder->new('TestShopPermanentDataForTestingUSPS.xml');


  SKIP:
    {
        ok( $TestBuilderUSPS->isShopImported(), 'Test Shop for USPS testing imported into DataBase' ) or
            skip('Persistent Tests for USPS skipped');

        my $ShippingFulfillmentMethod = $TestBuilderUSPS->getShippingFulfillmentMethod();
        my $ShippingLabel
          = $TestBuilderUSPS->getShopDependentObject('Customers/TestCustomer/Orders/5002/ShippingLabels/5002-1');

        my $PackagePickupAvailability
          = DE_EPAGES::USPS::API::WebServices::PackagePickupAvailability->new($ShippingFulfillmentMethod);
        $PackagePickupAvailability->feedRequest($ShippingLabel);

        ok( $PackagePickupAvailability->callWebService(),
            "$subTestName - called callWebService method on a filled request sourced from DB (5002-1 shipping label)" );
        ok(
            $PackagePickupAvailability->isValidResponseOrError(),
            "$subTestName - called isValidResponseOrError on a valid response"
        );

        $TestBuilderUSPS->cleanupTestEnvironment();
    }
    return;
}

#----------------------------------------------------------------------------------------
sub _feedAddressInfoObject {
    my ($PackagePickupAvailability) = @_;

    my $ShippingFulfillmentMethod = _createShippingFulfillmentMethod();
    $PackagePickupAvailability->setCredentials( $ShippingFulfillmentMethod->get('WebserviceUsername'),
                                                $ShippingFulfillmentMethod->get('WebservicePassword') );
    my $Address = _createAddress();
    $PackagePickupAvailability->setAddress($Address);
}

#----------------------------------------------------------------------------------------
sub _createAddress {
    my $hAddressAttributes = {
        'AddressExtension' => '',
        'City'             => 'GREENBELT',
        'Company'          => '',
        'State'            => 'MD',
        'Street'           => '6406 IVY LN',
        'Zipcode'          => '20770-1441',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Address::API::Object::Address', $hAddressAttributes );
}

#----------------------------------------------------------------------------------------
sub _createShippingFulfillmentMethod {
    my $hShippingFulfillmentMethodAttributes = {
        'WebserviceUsername'      => '605EPAGE1719',
        'WebservicePassword'      => '713BV14UD523',
        'Site'                    => _createMockShop(),
        'ShippingFulfillmentType' => _createShippingFulfillmentType(),
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::USPS::API::Object::ShippingFulfillmentMethodUSPS',
        $hShippingFulfillmentMethodAttributes );

}

#----------------------------------------------------------------------------------------
sub _createMockShop {
    my $hShopAttributes = { 'Alias' => 'Shop', 'IsInternalTestShop' => 1 };
    my $Shop = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Shop::API::Object::Shop', $hShopAttributes );
    return $Shop;
}

#----------------------------------------------------------------------------------------
sub _createShippingFulfillmentType {
    my $hShippingFulfillmentTypeAttributes = {
        'WebserviceURL'     => 'https://stg-secure.shippingapis.com',
        'WebserviceTestURL' => 'https://stg-secure.shippingapis.com'
    };
    my $ShippingFulfillmentType
      = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::ExternalShipping::API::Object::ShippingFulfillmentType',
        $hShippingFulfillmentTypeAttributes );
    return $ShippingFulfillmentType;
}

#=========================================================================================

RunScript( Sub => \&Main );

1;
