#========================================================================================
# package      DE_EPAGES::USPS::t::PackagePickupAvailability_internal
# state        public
#----------------------------------------------------------------------------------------
# description  internal integration tests for USPS Package Pickup Availability API
#               (Verify USPS tag)
#========================================================================================
package DE_EPAGES::USPS::t::PackagePickupAvailability_internal;

use strict;

use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::USPS::API::WebServices::PackagePickupAvailability;
use DE_EPAGES::USPS::t::PermanenceTestingBuilder;
use DE_EPAGES::USPS::t::MockedCalls qw ( RemoteClientMocker );
use Test::Exception;
use Test::More;
use File::Basename qw ( dirname );
use constant TEST_RESPONSES_DIRECTORY => dirname(__FILE__) . '/TestResponses/';

sub Main {

    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            eval {
                TestRequestValidationOnNoPersistence();
                TestRequestValidationOnPersistence();
            };
            if ( ExistsError() ) {
                my $Error = GetError();
                fail( 'Error: ' . $Error->longMessage );
            }

            done_testing();
        }
    );
}

#----------------------------------------------------------------------------------------
sub TestRequestValidationOnNoPersistence {
    my $subTestName = ( caller (0) )[3];

    my $ShippingFulfillmentMethod = _createShippingFulfillmentMethod();
    my $PackagePickupAvailability
      = DE_EPAGES::USPS::API::WebServices::PackagePickupAvailability->new($ShippingFulfillmentMethod);
    throws_ok(
        sub { $PackagePickupAvailability->isValidRequestOrError() },
        qr/INVALID_REQUEST_NOT_VALID_NODE_ATTRIBUTES/,
        "$subTestName - called isValidRequestOrError validator on a not filled request (no credentials)"
    );

    $ShippingFulfillmentMethod = _createShippingFulfillmentMethod();
    $PackagePickupAvailability->setCredentials( $ShippingFulfillmentMethod->get('WebserviceUsername'),
                                                $ShippingFulfillmentMethod->get('WebservicePassword')  );

    throws_ok( sub { $PackagePickupAvailability->isValidRequestOrError() },
        qr/STRING_EMPTY/,
        "$subTestName - called isValidRequestOrError validator on a not filled request (mandatory strings empty" );

    _feedPackagePickupAvailabilityObjectWithWrongAddress($PackagePickupAvailability);
    throws_ok(
        sub { $PackagePickupAvailability->isValidRequestOrError() },
        qr/Address2 is empty/,
        "$subTestName - Request without Address2"
    );

    _feedPackagePickupAvailabilityObject($PackagePickupAvailability);
    ok( $PackagePickupAvailability->isValidRequestOrError(),
        "$subTestName - called isValidRequestOrError validator on a filled request" );

    #Check positive responses
    my $Client
      = RemoteClientMocker( TEST_RESPONSES_DIRECTORY . 'PackagePickupAvailability_positive_response.xml', '200' );
    my $PackagePickupAvailabilityWithPositiveResponse
      = DE_EPAGES::USPS::API::WebServices::PackagePickupAvailability->new( $ShippingFulfillmentMethod, $Client );

    _feedPackagePickupAvailabilityObject($PackagePickupAvailabilityWithPositiveResponse);

    $PackagePickupAvailabilityWithPositiveResponse->callWebService();

    ok(
        $PackagePickupAvailabilityWithPositiveResponse->isValidResponseOrError(),
        "$subTestName - called isValidResponseOrError on a valid error"
    );

    is( $PackagePickupAvailabilityWithPositiveResponse->getFirmNameOrEmpty(),
        'XYZ Corp.', "$subTestName - called getFirmNameOrEmpty" );

    is( $PackagePickupAvailabilityWithPositiveResponse->getSuiteOrAptOrEmpty(),
        '', "$subTestName - called getSuiteOrAptOrEmpty" );

    is( $PackagePickupAvailabilityWithPositiveResponse->getAddress2OrError(),
        '6406 IVY LN', "$subTestName - called getAddress2OrError" );

    is( $PackagePickupAvailabilityWithPositiveResponse->getCityOrError(),
        'GREENBELT', "$subTestName - called getCityOrError" );

    is( $PackagePickupAvailabilityWithPositiveResponse->getStateOrError(),
        'MD', "$subTestName - called getStateOrError" );

    is( $PackagePickupAvailabilityWithPositiveResponse->getUrbanizationOrEmpty(),
        '', "$subTestName - called getUrbanizationOrEmpty" );

    is( $PackagePickupAvailabilityWithPositiveResponse->getZipcode5OrError(),
        '20770', "$subTestName - called getZipcode5OrError" );

    is( $PackagePickupAvailabilityWithPositiveResponse->getZipcode4OrEmpty(),
        '1441', "$subTestName - called getZipcode4OrEmpty" );

    is( $PackagePickupAvailabilityWithPositiveResponse->getDayOfWeekOrEmpty(),
        'Monday', "$subTestName - called getDayOfWeekOrEmpty" );

    is( $PackagePickupAvailabilityWithPositiveResponse->getDateOrEmpty(),
        '3/1/2004', "$subTestName - called getDateOrEmpty" );

    is( $PackagePickupAvailabilityWithPositiveResponse->getCarrierRouteOrEmpty(),
        'C', "$subTestName - called getCarrierRouteOrEmpty" );

    #Check negative responses
    $Client = RemoteClientMocker( TEST_RESPONSES_DIRECTORY . 'PackagePickupAvailability_negative_response.xml', '200' );

    my $PackagePickupAvailabilityWithErrorResponse
      = DE_EPAGES::USPS::API::WebServices::PackagePickupAvailability->new( $ShippingFulfillmentMethod, $Client );

    _feedPackagePickupAvailabilityObject($PackagePickupAvailabilityWithErrorResponse);

    $PackagePickupAvailabilityWithErrorResponse->callWebService();

    ok(
        $PackagePickupAvailabilityWithErrorResponse->isErrorResponse(),
        "$subTestName - called isErrorResponse validator on an erroneous response"
    );

    like( $PackagePickupAvailabilityWithErrorResponse->getErrorResponseDescriptionOrEmpty(),
        qr/^Invalid/, "$subTestName - called getErrorResponseDescriptionOrEmpty" );

    is( $PackagePickupAvailabilityWithErrorResponse->getErrorResponseNumberOrEmpty(),
        '-2147218002', "$subTestName - called getErrorResponseNumberOrEmpty" );

    is(
        $PackagePickupAvailabilityWithErrorResponse->getErrorResponseSourceOrEmpty(),
        'SOLServerPickupTest;SOLServerPickupTest.CarrierPickupAvailability_Respond',
        "$subTestName - called getErrorResponseSourceOrEmpty"
    );

    is( $PackagePickupAvailabilityWithErrorResponse->getErrorResponseHelpFileOrEmpty(),
        '', "$subTestName - called getErrorResponseHelpFileOrEmpty" );

    is( $PackagePickupAvailabilityWithErrorResponse->getErrorResponseHelpContextOrEmpty(),
        '', "$subTestName - called getErrorResponseHelpContextOrEmpty" );

    $Client = RemoteClientMocker( TEST_RESPONSES_DIRECTORY . 'Failed_Connection.txt', '500' );
    my $PackagePickupAvailabilityWithErrorConnection
      = DE_EPAGES::USPS::API::WebServices::PackagePickupAvailability->new( $ShippingFulfillmentMethod, $Client );

    _feedPackagePickupAvailabilityObject($PackagePickupAvailabilityWithErrorConnection);

    throws_ok( sub { $PackagePickupAvailabilityWithErrorConnection->callWebService() },
        qr/CONNECTION_FAILED/, "$subTestName - catched a Connection Failed error" );

}

#----------------------------------------------------------------------------------------
sub TestRequestValidationOnPersistence {
    my $subTestName = ( caller (0) )[3];

    my $TestBuilderUSPS    = DE_EPAGES::USPS::t::PermanenceTestingBuilder->new('TestShopPermanentDataForTestingUSPS.xml');

  SKIP:
    {
        ok( $TestBuilderUSPS->isShopImported(), 'Test Shop for USPS testing imported into DataBase' ) or
            skip('Persistent Tests for USPS skipped');

        my $ShippingFulfillmentMethod = $TestBuilderUSPS->getShippingFulfillmentMethod();
        my $ShippingLabel
          = $TestBuilderUSPS->getShopDependentObject('Customers/TestCustomer/Orders/5002/ShippingLabels/5002-1');
        my $Client
          = RemoteClientMocker( TEST_RESPONSES_DIRECTORY . 'PackagePickupAvailability_positive_response.xml', '200' );

        my $PackagePickupAvailability
          = DE_EPAGES::USPS::API::WebServices::PackagePickupAvailability->new( $ShippingFulfillmentMethod, $Client );
        $PackagePickupAvailability->feedRequest($ShippingLabel);

        ok(
            $PackagePickupAvailability->callWebService(),
            "$subTestName - called callWebService method on a filled request sourced from DB (5002-1 shipping label)"
        );

        $TestBuilderUSPS->cleanupTestEnvironment();
    }

    return;
}

#----------------------------------------------------------------------------------------
sub _feedPackagePickupAvailabilityObject {
    my ($PackagePickupAvailability) = @_;

    my $ShippingFulfillmentMethod = _createShippingFulfillmentMethod();
    $PackagePickupAvailability->setCredentials( $ShippingFulfillmentMethod->get('WebserviceUsername'),
                                                $ShippingFulfillmentMethod->get('WebservicePassword')  );
    my $Address = _createAddress();
    $PackagePickupAvailability->setAddress($Address);

}

#----------------------------------------------------------------------------------------
sub _createAddress {
    my $hAddressAttributes = {
        'AddressExtension' => '',
        'City'             => 'GREENBELT',
        'Company'          => 'XYZ Corp.',
        'State'            => 'MD',
        'Urbanization'     => '',
        'Street'           => '6406 IVY LN',
        'Zipcode'          => '20770-1441',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Address::API::Object::Address', $hAddressAttributes );
}

#----------------------------------------------------------------------------------------
sub _feedPackagePickupAvailabilityObjectWithWrongAddress {
    my ($PackagePickupAvailability) = @_;

    my $ShippingFulfillmentMethod = _createShippingFulfillmentMethod();
    $PackagePickupAvailability->setCredentials( $ShippingFulfillmentMethod->get('WebserviceUsername'),
                                                $ShippingFulfillmentMethod->get('WebservicePassword')  );

    my $Address = _createWrongAddress();
    $PackagePickupAvailability->setAddress($Address);

}

#----------------------------------------------------------------------------------------
sub _createWrongAddress {
    my $hAddressAttributes = {
        'AddressExtension' => '',
        'City'             => 'GREENBELT',
        'Company'          => 'XYZ Corp.',
        'State'            => 'MD',
        'Urbanization'     => '',
        'Street'           => '',
        'Zipcode'          => '20770-1441',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Address::API::Object::Address', $hAddressAttributes );
}

#----------------------------------------------------------------------------------------
sub _createShippingFulfillmentMethod {
    my $hShippingFulfillmentMethodAttributes = {
        'WebserviceUsername'      => '605EPAGE1719',
        'WebservicePassword'      => '713BV14UD523',
        'Site'                    => _createMockShop(),
        'ShippingFulfillmentType' => _createShippingFulfillmentType(),
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::USPS::API::Object::ShippingFulfillmentMethodUSPS',
        $hShippingFulfillmentMethodAttributes );

}

#----------------------------------------------------------------------------------------
sub _createMockShop {
    my $hShopAttributes = { 'Alias' => 'Shop', 'IsInternalTestShop' => 1 };
    my $Shop = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Shop::API::Object::Shop', $hShopAttributes );
    return $Shop;
}

#----------------------------------------------------------------------------------------
sub _createShippingFulfillmentType {
    my $hShippingFulfillmentTypeAttributes = {
        'WebserviceURL'     => 'https://stg-secure.shippingapis.com',
        'WebserviceTestURL' => 'https://stg-secure.shippingapis.com'
    };
    my $ShippingFulfillmentType
      = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::ExternalShipping::API::Object::ShippingFulfillmentType',
        $hShippingFulfillmentTypeAttributes );
    return $ShippingFulfillmentType;
}

RunScript( Sub => \&Main );

1;
