#========================================================================================
# package      DE_EPAGES::USPS::t::PackagePickupCancel_external
# state        public
#----------------------------------------------------------------------------------------
# description  external integration tests for USPS Package Pickup Cancel API
#               (CarrierPickupCancel USPS tag)
#========================================================================================
package DE_EPAGES::USPS::t::PackagePickupCancel_external;

use strict;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use Test::Exception;
use Test::More;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::Core::API::PerlTools qw ( GetLocalTimeZone );

use DE_EPAGES::USPS::API::WebServices::PackagePickupCancel;
use DE_EPAGES::USPS::API::WebServices::PackagePickupSchedule;
use DE_EPAGES::USPS::t::PermanenceTestingBuilder;

sub Main {

    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            if ( $ENV{SKIP_EXTERNAL_TESTS} ) {
                plan skip_all => 'call USPS Server (SKIP_EXTERNAL_TESTS == 1)';
            }

            eval {
                TestRequestValidationOnNoPersistence();
                TestRequestValidationOnPersistence();
            };
            if ( ExistsError() ) {
                my $Error = GetError();
                fail( 'Error: ' . $Error->longMessage );
            }

            done_testing();
        }
    );
}

#----------------------------------------------------------------------------------------
sub TestRequestValidationOnNoPersistence {
    my $subTestName = ( caller (0) )[3];

    my $ShippingFulfillmentMethod = _createShippingFulfillmentMethod();
    my $PackagePickupSchedule
      = DE_EPAGES::USPS::API::WebServices::PackagePickupSchedule->new($ShippingFulfillmentMethod);

    # Schedule pickup creation to can cancel it afterwards
    _feedPackagePickupScheduleObject($PackagePickupSchedule);
    ok( $PackagePickupSchedule->isValidRequestOrError(), "$subTestName - Pickup Scheduled request OK" );

    $PackagePickupSchedule->callWebService();

    ok( $PackagePickupSchedule->isValidResponseOrError(), "$subTestName - Pickup Scheduled OK" );

    my $PetitionNumber = $PackagePickupSchedule->getConfirmationNumberOrEmpty();

    isnt( $PetitionNumber, '', "$subTestName - Petition number not empty" );

    #Cancelation of the previous Scheduled Pickup
    my $PackagePickupCancel = DE_EPAGES::USPS::API::WebServices::PackagePickupCancel->new($ShippingFulfillmentMethod);

    _setTestRequestValues($PackagePickupCancel);
    $PackagePickupCancel->setConfirmationNumber($PetitionNumber);

    ok( $PackagePickupCancel->isValidRequestOrError(), "$subTestName - Cancel request OK" );

    $PackagePickupCancel->callWebService();

    ok( $PackagePickupCancel->isValidResponseOrError(),
        "$subTestName - called isValidResponseOrError validator on a response" )

}

#----------------------------------------------------------------------------------------
sub _setTestRequestValues {
    my ($PackagePickupCancel) = @_;

    my $ShippingFulfillmentMethod = _createShippingFulfillmentMethod();
    $PackagePickupCancel->setCredentials( $ShippingFulfillmentMethod->get('WebserviceUsername'),
                                          $ShippingFulfillmentMethod->get('WebservicePassword') );

    my $Address = _createAddress();
    $PackagePickupCancel->setAddress($Address);

}

#----------------------------------------------------------------------------------------
sub _feedPackagePickupScheduleObject {
    my ($PackagePickupSchedule) = @_;

    my $ShippingFulfillmentMethod = _createShippingFulfillmentMethod();
    $PackagePickupSchedule->setCredentials( $ShippingFulfillmentMethod->get('WebserviceUsername'),
                                            $ShippingFulfillmentMethod->get('WebservicePassword') );

    my $OriginAddress = _createAddress();
    my $ShippingLabel = _createShippingLabel();

    $PackagePickupSchedule->setAddress($OriginAddress);
    $PackagePickupSchedule->setPickupSettings($ShippingLabel);
}

#----------------------------------------------------------------------------------------
sub _createShippingFulfillmentMethod {
    my $hShippingFulfillmentMethodAttributes = {
        'WebserviceUsername'      => '605EPAGE1719',
        'WebservicePassword'      => '713BV14UD523',
        'Site'                    => _createMockShop(),
        'ShippingFulfillmentType' => _createShippingFulfillmentType(),
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::USPS::API::Object::ShippingFulfillmentMethodUSPS',
        $hShippingFulfillmentMethodAttributes );

}

#----------------------------------------------------------------------------------------
sub _createMockShop {
    my $hShopAttributes = { 'Alias' => 'Shop', 'IsInternalTestShop' => 1 };
    my $Shop = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Shop::API::Object::Shop', $hShopAttributes );
    return $Shop;
}

#----------------------------------------------------------------------------------------
sub _createShippingFulfillmentType {
    my $hShippingFulfillmentTypeAttributes = {
        'WebserviceURL'     => 'https://stg-secure.shippingapis.com',
        'WebserviceTestURL' => 'https://stg-secure.shippingapis.com'
    };
    my $ShippingFulfillmentType
      = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::ExternalShipping::API::Object::ShippingFulfillmentType',
        $hShippingFulfillmentTypeAttributes );
    return $ShippingFulfillmentType;
}

#----------------------------------------------------------------------------------------
sub _createAddress {
    my $hAddressAttributes = {
        'AddressExtension' => 'RM 2100',
        'City'             => 'Washington',
        'Company'          => 'ePages',
        'FirstName'        => 'John',
        'LastName'         => 'Doe',
        'Phone'            => '001 1020268',
        'Fullname'         => 'John Doe',
        'State'            => 'DC',
        'Street'           => '144 Constitution Ave NE',
        'Zipcode'          => '20260',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Address::API::Object::Address', $hAddressAttributes );
}

#----------------------------------------------------------------------------------------
sub _createShippingLabel {
    my $hLabelAttributes = {
        'ImageType'                  => 'None',
        'ParcelDepth'                => '5.30',
        'ParcelLength'               => '5.20',
        'ParcelWidth'                => '10.30',
        'ParcelGirth'                => '8.70',
        'PickupDate'                 => DateTime->today( time_zone => GetLocalTimeZone() ),
        'ShippingFulfillmentProduct' => _createShippingFulfillmentProduct(),
        'SizeUnit'                   => _createInchUnitOfMeasurement(),
        'TotalWeight'                => '10',
        'WeightUnit'                 => _createOunceUnitOfMeasurement(),
        'ServiceType'                => _createServiceType(),
        'PackageLocation'            => _createPackageLocation(),
        'SpecialInstructions'        => 'ring the bell',
    };

    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::USPS::API::Object::ShippingLabelUSPS',
        $hLabelAttributes );
}

#----------------------------------------------------------------------------------------
sub _createServiceType {
    my $hServiceTypeAttributes = { 'Alias' => 'PRIORITY_MAIL', 'USPSWebservice2ndKey' => 'PriorityMail' };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::USPS::API::Object::ListItemUSPS',
        $hServiceTypeAttributes );
}

#----------------------------------------------------------------------------------------
sub _createPackageLocation {
    my $hPackageLocationAttributes = { 'Alias' => 'BACK_DOOR ', 'USPSWebserviceKey' => 'Back door' };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::USPS::API::Object::ListItemUSPS',
        $hPackageLocationAttributes );
}

#----------------------------------------------------------------------------------------
sub _createShippingFulfillmentProduct {
    my $hProductAttributes = { 'Alias' => 'PriorityMail', };
    return DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::ExternalShipping::API::Object::ShippingFulfillmentProduct',
        $hProductAttributes );
}

#----------------------------------------------------------------------------------------
sub _createInchUnitOfMeasurement {
    my $hMetreAttributes = {
        'Alias' => 'metre',
        'Path'  => '/Units/metre',
    };
    my $metre = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Object::API::Object::UnitOfMeasurement',
        $hMetreAttributes );
    my $hInchAttributes = {
        'Alias'  => 'inch',
        'Factor' => 0.0254,
        'Parent' => $metre,
        'Path'   => '/Units/metre/inch',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Object::API::Object::UnitOfMeasurement',
        $hInchAttributes );
}

#----------------------------------------------------------------------------------------
sub _createOunceUnitOfMeasurement {
    my $hGramAttributes = { 'Alias' => 'gram' };
    my $Gram = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Object::API::Object::UnitOfMeasurement',
        $hGramAttributes );
    my $hOunceAttributes = {
        'Alias'  => 'ounce',
        'Factor' => 28.3495,
        'Parent' => $Gram,
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Object::API::Object::UnitOfMeasurement',
        $hOunceAttributes );
}

#----------------------------------------------------------------------------------------
sub TestRequestValidationOnPersistence {
    my $subTestName = ( caller (0) )[3];

    my $TestBuilderUSPS    = DE_EPAGES::USPS::t::PermanenceTestingBuilder->new('TestShopPermanentDataForTestingUSPS.xml');

  SKIP:
    {
        ok( $TestBuilderUSPS->isShopImported(), 'Test Shop for USPS testing imported into DataBase' ) or
            skip('Persistent Tests for USPS skipped');

        my $ShippingFulfillmentMethod = $TestBuilderUSPS->getShippingFulfillmentMethod();
        my $ShippingLabel
          = $TestBuilderUSPS->getShopDependentObject('Customers/TestCustomer/Orders/5002/ShippingLabels/5002-2');

        my $PackagePickupSchedule
          = DE_EPAGES::USPS::API::WebServices::PackagePickupSchedule->new($ShippingFulfillmentMethod);
        $PackagePickupSchedule->feedRequest($ShippingLabel);

        ok( $PackagePickupSchedule->callWebService(),
            "$subTestName - called PackagePickupSchedule-callWebService method on a filled request sourced from DB (5002-2 shipping label)"
        );
        ok( $PackagePickupSchedule->isValidResponseOrError(),
            "$subTestName - called PackagePickupSchedule-isValidResponseOrError on a valid response" );
        my $ConfirmationNumber = $PackagePickupSchedule->getConfirmationNumberOrEmpty();
        $ShippingLabel->set( { 'ConfirmationNumber' => $ConfirmationNumber, 'IsPickUpRequested' => 1, } );

        my $PackagePickupCancel
          = DE_EPAGES::USPS::API::WebServices::PackagePickupCancel->new($ShippingFulfillmentMethod);
        $PackagePickupCancel->feedRequest($ShippingLabel);

        ok( $PackagePickupCancel->callWebService(),
            "$subTestName - called PackagePickupCancel-callWebService method on a filled request sourced from DB (5002-2 shipping label)"
        );
        ok( $PackagePickupCancel->isValidResponseOrError(),
            "$subTestName - called PackagePickupCancel-isValidResponseOrError on a valid response" );

        $TestBuilderUSPS->cleanupTestEnvironment();
    }

    return;
}

#=========================================================================================

RunScript( Sub => \&Main );

1;
