#========================================================================================
# package      DE_EPAGES::USPS::t::PackagePickupCancel_internal
# state        public
#----------------------------------------------------------------------------------------
# description  internal integration tests for USPS Package Pickup Cancel API
#               (CarrierPickupCancel USPS tag)
#========================================================================================
package DE_EPAGES::USPS::t::PackagePickupCancel_internal;

use strict;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::USPS::t::MockedCalls qw ( RemoteClientMocker );
use Test::Exception;
use Test::More;
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::USPS::API::WebServices::PackagePickupCancel;
use DE_EPAGES::USPS::t::PermanenceTestingBuilder;
use File::Basename qw ( dirname );

use constant TEST_RESPONSES_DIRECTORY  => dirname(__FILE__) . '/TestResponses/';


sub Main {

    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            eval {
                TestRequestValidationOnNoPersistence();
                TestRequestValidationOnPersistence();
            };
            if ( ExistsError() ) {
                my $Error = GetError();
                fail( 'Error: ' . $Error->longMessage );
            }

            done_testing();
        }
    );
}

#----------------------------------------------------------------------------------------
sub TestRequestValidationOnNoPersistence {
    my $subTestName = ( caller (0) )[3];

    my $Client;

    my $ShippingFulfillmentMethod = _createShippingFulfillmentMethod();

    # Check the Requests

    # Check what happens if the Request has no USERID (from CarrierPickupCancelRequest) value
    my $PackagePickupCancel = DE_EPAGES::USPS::API::WebServices::PackagePickupCancel->new($ShippingFulfillmentMethod);
    throws_ok(
        sub { $PackagePickupCancel->isValidRequestOrError() },
        qr/INVALID_REQUEST_NOT_VALID_NODE_ATTRIBUTES/,
        "$subTestName - Request without Credentials - <CarrierPickupCancelRequest USERID=\"\"..."
    );

    $ShippingFulfillmentMethod = _createShippingFulfillmentMethod();
    $PackagePickupCancel->setCredentials( $ShippingFulfillmentMethod->get('WebserviceUsername'),
                                          $ShippingFulfillmentMethod->get('WebservicePassword') );

    throws_ok( sub { $PackagePickupCancel->isValidRequestOrError() },
        qr/STRING_EMPTY/,
        "$subTestName - called isValidRequestOrError validator on a not filled request (mandatory strings empty)" );

    # Check what happens if the Request has no Address2 value (which is mandatory)
    _feedPackagePickupCancelObjectWithWrongAddress($PackagePickupCancel);
    throws_ok(
        sub { $PackagePickupCancel->isValidRequestOrError() },
        qr/Address2 is empty/,
        "$subTestName - Request without Address2"
    );

    # Check what happens if the Request is well formatted and all values are there
    _feedPackagePickupCancelObject($PackagePickupCancel);
    ok( $PackagePickupCancel->isValidRequestOrError(), "$subTestName - Good Request" );

    # Check the (mocked) Responses

    # Check what happens if we obtain a ResponseCode different than HTTP_CODE_OK (200)
    $Client = RemoteClientMocker( TEST_RESPONSES_DIRECTORY . 'Failed_Connection.txt', '500' );
    my $PackagePickupCancelConnectionFailed
      = DE_EPAGES::USPS::API::WebServices::PackagePickupCancel->new( $ShippingFulfillmentMethod, $Client );

    _feedPackagePickupCancelObject($PackagePickupCancelConnectionFailed);

    throws_ok( sub { $PackagePickupCancelConnectionFailed->callWebService() },
        qr/CONNECTION_FAILED/, "$subTestName - ResponseCode != HTTP_CODE_OK (200)" );

    #Check error responses
    $Client = RemoteClientMocker( TEST_RESPONSES_DIRECTORY . 'PackagePickupCancel_negative_response.xml', '200' );

    my $PackagePickupCancelErrorResponse
      = DE_EPAGES::USPS::API::WebServices::PackagePickupCancel->new( $ShippingFulfillmentMethod, $Client );

    _feedPackagePickupCancelObject($PackagePickupCancelErrorResponse);

    $PackagePickupCancelErrorResponse->callWebService();

    ok( $PackagePickupCancelErrorResponse->isErrorResponse(), "$subTestName - Response has an Error tag" );

    like( $PackagePickupCancelErrorResponse->getErrorResponseDescriptionOrEmpty(),
        qr/^Invalid/, "$subTestName - called getErrorResponseDescriptionOrEmpty" );

    is( $PackagePickupCancelErrorResponse->getErrorResponseNumberOrEmpty(),
        '-2147218002', "$subTestName - called getErrorResponseNumberOrEmpty" );

    is(
        $PackagePickupCancelErrorResponse->getErrorResponseSourceOrEmpty(),
        'SOLServerPickupTest;SOLServerPickupTest.CarrierPickupCancel_Respond',
        "$subTestName - called getErrorResponseSourceOrEmpty"
    );

    is( $PackagePickupCancelErrorResponse->getErrorResponseHelpFileOrEmpty(),
        '', "$subTestName - called getErrorResponseHelpFileOrEmpty" );

    is( $PackagePickupCancelErrorResponse->getErrorResponseHelpContextOrEmpty(),
        '', "$subTestName - called getErrorResponseHelpContextOrEmpty" );

    # Check Positive response
    $Client = RemoteClientMocker( TEST_RESPONSES_DIRECTORY . 'PackagePickupCancel_positive_response.xml', '200' );
    my $PackagePickupCancelPositiveResponse
      = DE_EPAGES::USPS::API::WebServices::PackagePickupCancel->new( $ShippingFulfillmentMethod, $Client );
    _feedPackagePickupCancelObject($PackagePickupCancelPositiveResponse);
    $PackagePickupCancelPositiveResponse->callWebService();
    ok( $PackagePickupCancelPositiveResponse->isValidResponseOrError(), "$subTestName - Response isValid" );
    is( $PackagePickupCancelPositiveResponse->getFirmNameOrEmpty(), 'ABC CORP.', "$subTestName - Can get FirmName" );
    is( $PackagePickupCancelPositiveResponse->getSuiteOrAptOrEmpty(), 'SUITE 777',
        "$subTestName - Can get SuiteOrApt" );
    is(
        $PackagePickupCancelPositiveResponse->getAddress2OrError(),
        '1390 MARKET STREET',
        "$subTestName - Can get Address2"
    );
    is( $PackagePickupCancelPositiveResponse->getUrbanizationOrEmpty(), '', "$subTestName - Can get Urbanization" );
    is( $PackagePickupCancelPositiveResponse->getCityOrError(),     'HOUSTON', "$subTestName - Can get City" );
    is( $PackagePickupCancelPositiveResponse->getStateOrError(),    'TX',      "$subTestName - Can get State" );
    is( $PackagePickupCancelPositiveResponse->getZipcode5OrError(), '77058',   "$subTestName - Can get Zipcode5" );
    is( $PackagePickupCancelPositiveResponse->getZipcode4OrEmpty(), '1234',    "$subTestName - Can get Zipcode4" );
    is( $PackagePickupCancelPositiveResponse->getConfirmationNumberOrEmpty(),
        'WTC123456789', "$subTestName - Can get ConfirmationNumber" );
    like(
        $PackagePickupCancelPositiveResponse->getStatusOrEmpty(),
        qr/Your pickup request/,
        "$subTestName - Can get Status"
    );

    return;
}

#----------------------------------------------------------------------------------------
sub _feedPackagePickupCancelObject {
    my ($PackagePickupCancel) = @_;

    my $ShippingFulfillmentMethod = _createShippingFulfillmentMethod();
    $PackagePickupCancel->setCredentials( $ShippingFulfillmentMethod->get('WebserviceUsername'),
                                          $ShippingFulfillmentMethod->get('WebservicePassword') );

    my $Address = _createAddress();
    $PackagePickupCancel->setAddress($Address);

    $PackagePickupCancel->setConfirmationNumber('WTC57827596');

}

#----------------------------------------------------------------------------------------
sub _createShippingFulfillmentMethod {
    my $hShippingFulfillmentMethodAttributes = {
        'WebserviceUsername'      => '605EPAGE1719',
        'WebservicePassword'      => '713BV14UD523',
        'Site'                    => _createMockShop(),
        'ShippingFulfillmentType' => _createShippingFulfillmentType(),
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::USPS::API::Object::ShippingFulfillmentMethodUSPS',
        $hShippingFulfillmentMethodAttributes );

}

#----------------------------------------------------------------------------------------
sub _createMockShop {
    my $hShopAttributes = { 'Alias' => 'Shop', 'IsInternalTestShop' => 1 };
    my $Shop = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Shop::API::Object::Shop', $hShopAttributes );
    return $Shop;
}

#----------------------------------------------------------------------------------------
sub _createShippingFulfillmentType {
    my $hShippingFulfillmentTypeAttributes = {
        'WebserviceURL'     => 'https://stg-secure.shippingapis.com',
        'WebserviceTestURL' => 'https://stg-secure.shippingapis.com'
    };
    my $ShippingFulfillmentType
      = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::ExternalShipping::API::Object::ShippingFulfillmentType',
        $hShippingFulfillmentTypeAttributes );
    return $ShippingFulfillmentType;
}

#----------------------------------------------------------------------------------------
sub _createAddress {
    my $hAddressAttributes = {
        'AddressExtension' => 'Suite 777',
        'City'             => 'Houston',
        'Company'          => 'ABC Corp.',
        'State'            => 'TX',
        'Urbanization'     => '',
        'Street'           => '1390 Market Street',
        'Zipcode'          => '77058-1234',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Address::API::Object::Address', $hAddressAttributes );
}

#----------------------------------------------------------------------------------------
sub _feedPackagePickupCancelObjectWithWrongAddress {
    my ($PackagePickupCancel) = @_;

    my $ShippingFulfillmentMethod = _createShippingFulfillmentMethod();
    $PackagePickupCancel->setCredentials( $ShippingFulfillmentMethod->get('WebserviceUsername'),
                                          $ShippingFulfillmentMethod->get('WebservicePassword') );

    my $Address = _createWrongAddress();
    $PackagePickupCancel->setAddress($Address);

    $PackagePickupCancel->setConfirmationNumber('WTC57827596');
}

#----------------------------------------------------------------------------------------
sub _createWrongAddress {
    my $hAddressAttributes = {
        'AddressExtension' => 'Suite 777',
        'City'             => 'Houston',
        'Company'          => 'ABC Corp.',
        'State'            => 'TX',
        'Urbanization'     => '',
        'Street'           => '',
        'Zipcode'          => '77058-1234',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Address::API::Object::Address', $hAddressAttributes );
}

#----------------------------------------------------------------------------------------
sub TestRequestValidationOnPersistence {
    my $subTestName = ( caller (0) )[3];

    my $TestBuilderUSPS = DE_EPAGES::USPS::t::PermanenceTestingBuilder->new('TestShopPermanentDataForTestingUSPS.xml');

    SKIP:
    {
        ok($TestBuilderUSPS->isShopImported(), 'Test Shop for USPS testing imported into DataBase') or
            skip( 'Persistent Tests for USPS skipped' );

        my $ShippingFulfillmentMethod = $TestBuilderUSPS->getShippingFulfillmentMethod();
        my $ShippingLabel =
            $TestBuilderUSPS->getShopDependentObject('Customers/TestCustomer/Orders/5002/ShippingLabels/5002-1');
        my $Client
          = RemoteClientMocker( TEST_RESPONSES_DIRECTORY . 'PackagePickupCancel_positive_response.xml', '200' );

        my $PackagePickupCancel
          = DE_EPAGES::USPS::API::WebServices::PackagePickupCancel->new( $ShippingFulfillmentMethod, $Client );
        $PackagePickupCancel->feedRequest($ShippingLabel);

        ok( $PackagePickupCancel->callWebService(),
            "$subTestName - called callWebService method on a filled request sourced from DB (5002-1 shipping label)" );
        ok( $PackagePickupCancel->isValidResponseOrError(),
            "$subTestName - called isValidResponseOrError on a valid response" );

        $TestBuilderUSPS->cleanupTestEnvironment();
    }

    return;
}


#=========================================================================================

RunScript( Sub => \&Main );

1;
