#========================================================================================
# package      DE_EPAGES::USPS::t::PackagePickupSchedule_internal
# state        public
#----------------------------------------------------------------------------------------
# description  internal integration tests for USPS PackagePickupSchedule API
#========================================================================================
package DE_EPAGES::USPS::t::PackagePickupSchedule_internal;

use strict;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw ( GetLocalTimeZone );
use DE_EPAGES::USPS::t::MockedCalls qw ( RemoteClientMocker );
use DE_EPAGES::USPS::API::WebServices::PackagePickupSchedule;
use DE_EPAGES::USPS::t::PermanenceTestingBuilder;
use DE_EPAGES::Test::Mock::EpagesObject;
use Test::More;
use Test::Exception;
use File::Basename qw ( dirname );

use constant TEST_RESPONSES_DIRECTORY =>  dirname(__FILE__) . '/TestResponses/';


sub Main {

    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            eval {
                TestRequestValidationOnNoPersistence();
                TestRequestValidationOnPersistence();
            };
            if ( ExistsError() ) {
                my $Error = GetError();
                fail( 'Error: ' . $Error->longMessage );
            }
            done_testing();
        }
    );
}

#----------------------------------------------------------------------------------------
sub TestRequestValidationOnNoPersistence {
    my $subTestName = ( caller (0) )[3];

    my $ShippingFulfillmentMethod = _createShippingFulfillmentMethod();
    # request validation tests
    my $PackagePickupSchedule = DE_EPAGES::USPS::API::WebServices::PackagePickupSchedule->new($ShippingFulfillmentMethod);

    throws_ok( sub { $PackagePickupSchedule->isValidRequestOrError() },
        qr/INVALID_REQUEST_NOT_VALID_NODE_ATTRIBUTES/,
        "$subTestName - called isValidRequestOrError validator on a not filled request (no credentials)"
    );

    $ShippingFulfillmentMethod = _createShippingFulfillmentMethod();
    $PackagePickupSchedule->setCredentials($ShippingFulfillmentMethod->get('WebserviceUsername'),
                                           $ShippingFulfillmentMethod->get('WebservicePassword'));

    throws_ok( sub { $PackagePickupSchedule->isValidRequestOrError() },
        qr/STRING_EMPTY/,
        "$subTestName - called isValidRequestOrError validator on a not filled request (mandatory strings empty"
    );

    _feedPackagePickupScheduleObject($PackagePickupSchedule);
    ok( $PackagePickupSchedule->isValidRequestOrError(),
        "$subTestName - called isValidRequestAndInternalConditionsOrError validator on a filled request" );

    _feedPackagePickupScheduleObjectWithWrongAddress($PackagePickupSchedule);
    throws_ok( sub { $PackagePickupSchedule->isValidRequestOrError() },
        qr/Address2 is empty/,
        "$subTestName - Request without Address2"
    );

    #positive response tests

    my $Client = RemoteClientMocker(
         TEST_RESPONSES_DIRECTORY . 'PackagePickupSchedule_positive_response.xml',
        '200'
    );

    my $PackagePickupSchedulePositive = DE_EPAGES::USPS::API::WebServices::PackagePickupSchedule->new($ShippingFulfillmentMethod,$Client);
    _feedPackagePickupScheduleObject($PackagePickupSchedulePositive);
    $PackagePickupSchedulePositive->callWebService();

    ok( $PackagePickupSchedulePositive->isValidResponseOrError(),
        "$subTestName - called isValidResponseOrError on a valid response"
    );

    is( $PackagePickupSchedulePositive->getFirmNameOrEmpty(),
        'ePages',
        "$subTestName - called getFirmNameOrEmpty"
    );

    is( $PackagePickupSchedulePositive->getFirstNameOrEmpty(),
        'John',
        "$subTestName - called getFirstNameOrEmpty"
    );

    is( $PackagePickupSchedulePositive->getLastNameOrEmpty(),
        'Doe',
        "$subTestName - called getLastNameOrEmpty"
    );

    is( $PackagePickupSchedulePositive->getSuiteOrAptOrEmpty(),
        'RM 2100',
        "$subTestName - called getSuiteOrAptOrEmpty"
    );

    is( $PackagePickupSchedulePositive->getAddress2OrError(),
        '475 Enfant Plaza SW',
        "$subTestName - called getAddress2OrError"
    );

    is( $PackagePickupSchedulePositive->getCityOrError(),
        'Washington',
        "$subTestName - called getCityOrError"
    );

    is( $PackagePickupSchedulePositive->getStateOrError(),
        'DC',
        "$subTestName - called getStateOrError"
    );

    is( $PackagePickupSchedulePositive->getUrbanizationOrEmpty(),
        '',
        "$subTestName - called getUrbanizationOrEmpty"
    );

    is( $PackagePickupSchedulePositive->getZipcode5OrError(),
        '20260',
        "$subTestName - called getZipcode5OrError"
    );

    is( $PackagePickupSchedulePositive->getZipcode4OrEmpty(),
        '',
        "$subTestName - called getZipcode4OrEmpty"
    );

    is( $PackagePickupSchedulePositive->getPhoneOrEmpty(),
        '123456789',
        "$subTestName - called getPhoneOrEmpty"
    );

    is( $PackagePickupSchedulePositive->getExtensionOrEmpty(),
        '',
        "$subTestName - called getExtensionOrEmpty"
    );

    is( $PackagePickupSchedulePositive->getDayOfWeekOrEmpty(),
        'Monday',
        "$subTestName - called getDayOfWeekOrEmpty"
    );

    is( $PackagePickupSchedulePositive->getDateOrEmpty(),
        '04/01/2004',
        "$subTestName - called getDateOrEmpty"
    );

    is( $PackagePickupSchedulePositive->getEstimatedWeightOrEmpty(),
        '10',
        "$subTestName - called getEstimatedWeightOrEmpty"
    );

    is( $PackagePickupSchedulePositive->getPackageLocationOrEmpty(),
        'Other',
        "$subTestName - called getPackageLocationOrEmpty"
    );

    is( $PackagePickupSchedulePositive->getSpecialInstructionsOrEmpty(),
        '',
        "$subTestName - called getSpecialInstructionsOrEmpty"
    );

    is( $PackagePickupSchedulePositive->getPackageOrEmpty(),
        '1',
        "$subTestName - called getPackageOrEmpty"
    );

    is( $PackagePickupSchedulePositive->getConfirmationNumberOrEmpty(),
        'ABC12345',
        "$subTestName - called getConfirmationNumberOrEmpty"
    );

    is( $PackagePickupSchedulePositive->getEmailAddressOrEmpty(),
        '',
        "$subTestName - called getEmailAddressOrEmpty"
    );

    is( $PackagePickupSchedulePositive->getCarrierRouteOrEmpty(),
        'C',
        "$subTestName - called getCarrierRouteOrEmpty"
    );

    is( $PackagePickupSchedulePositive->getServiceTypeOrEmpty(),
        'PriorityMail',
        "$subTestName - called getServiceTypeOrEmpty"
    );

    is( $PackagePickupSchedulePositive->getCountTypeOrEmpty(),
        '1',
        "$subTestName - called getCountTypeOrEmpty"
    );

    #error response tests

    $Client = RemoteClientMocker(
        TEST_RESPONSES_DIRECTORY . 'PackagePickupSchedule_negative_response.xml',
        '200'
    );
    my $PackagePickupScheduleNegative = DE_EPAGES::USPS::API::WebServices::PackagePickupSchedule->new($ShippingFulfillmentMethod,$Client);
    _feedPackagePickupScheduleObject($PackagePickupScheduleNegative);

    $PackagePickupScheduleNegative->callWebService();
    ok( $PackagePickupScheduleNegative->isErrorResponse(),
        "$subTestName - called isErrorResponse validator on an erroneus response"
    );

    like( $PackagePickupScheduleNegative->getErrorResponseDescriptionOrEmpty(),
        qr/^Invalid/, "$subTestName - called getErrorResponseDescriptionOrEmpty" );

    is( $PackagePickupScheduleNegative->getErrorResponseNumberOrEmpty(),
        '-2147218002', "$subTestName - called getErrorResponseNumberOrEmpty" );

    is( $PackagePickupScheduleNegative->getErrorResponseSourceOrEmpty(),
        'SOLServerPickupTest;SOLServerPickupTest.CarrierPickupSchedule_Respond', "$subTestName - called getErrorResponseSourceOrEmpty" );

    is( $PackagePickupScheduleNegative->getErrorResponseHelpFileOrEmpty(),
        '', "$subTestName - called getErrorResponseHelpFileOrEmpty" );

    is( $PackagePickupScheduleNegative->getErrorResponseHelpContextOrEmpty(),
        '', "$subTestName - called getErrorResponseHelpContextOrEmpty" );

    $Client = RemoteClientMocker(
        TEST_RESPONSES_DIRECTORY . 'Failed_Connection.txt', '500'
    );
    my $PackagePickupScheduleNoConnection = DE_EPAGES::USPS::API::WebServices::PackagePickupSchedule->new($ShippingFulfillmentMethod,$Client);
    _feedPackagePickupScheduleObject($PackagePickupScheduleNoConnection);
    throws_ok( sub { $PackagePickupScheduleNoConnection->callWebService() },
        qr/CONNECTION_FAILED/,
        "$subTestName - catched a Connection Failed error"
    );
}

#----------------------------------------------------------------------------------------
sub _createShippingFulfillmentMethod {
    my $hShippingFulfillmentMethodAttributes = {
        'WebserviceUsername' => '605EPAGE1719',
        'WebservicePassword'    => '713BV14UD523',
        'Site' => _createMockShop(),
        'ShippingFulfillmentType' => _createShippingFulfillmentType(),
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::USPS::API::Object::ShippingFulfillmentMethodUSPS',
        $hShippingFulfillmentMethodAttributes );

}
#----------------------------------------------------------------------------------------
sub _createMockShop {
    my $hShopAttributes = { 'Alias' => 'Shop', 'IsInternalTestShop' => 1 };
    my $Shop = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::Shop::API::Object::Shop',
        $hShopAttributes );
    return $Shop;
}
#----------------------------------------------------------------------------------------
sub _createShippingFulfillmentType {
    my $hShippingFulfillmentTypeAttributes = { 'WebserviceURL' => 'https://stg-secure.shippingapis.com',
                                               'WebserviceTestURL' => 'https://stg-secure.shippingapis.com' };
    my $ShippingFulfillmentType = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::ExternalShipping::API::Object::ShippingFulfillmentType',
        $hShippingFulfillmentTypeAttributes );
    return $ShippingFulfillmentType;
}
#----------------------------------------------------------------------------------------
sub TestRequestValidationOnPersistence {
    my $subTestName = ( caller (0) )[3];

    my $TestBuilderUSPS = DE_EPAGES::USPS::t::PermanenceTestingBuilder->new('TestShopPermanentDataForTestingUSPS.xml');

    SKIP:
    {
        ok($TestBuilderUSPS->isShopImported(), 'Test Shop for USPS testing imported into DataBase') or
            skip( 'Persistent Tests for USPS skipped' );

        my $ShippingFulfillmentMethod = $TestBuilderUSPS->getShippingFulfillmentMethod();
        my $ShippingLabel =
            $TestBuilderUSPS->getShopDependentObject('Customers/TestCustomer/Orders/5002/ShippingLabels/5002-1');
        my $Client = RemoteClientMocker(
             TEST_RESPONSES_DIRECTORY . 'PackagePickupSchedule_positive_response.xml',
            '200'
        );

        my $PackagePickupSchedule =
            DE_EPAGES::USPS::API::WebServices::PackagePickupSchedule->new($ShippingFulfillmentMethod, $Client);
        $PackagePickupSchedule->feedRequest($ShippingLabel);

        ok( $PackagePickupSchedule->callWebService(),
            "$subTestName - called callWebService method on a filled request sourced from DB (5002-1 shipping label)"
        );
        ok( $PackagePickupSchedule->isValidResponseOrError(),
            "$subTestName - called isValidResponseOrError method validator on a valid response"
        );

        $TestBuilderUSPS->cleanupTestEnvironment();
    }

    return;
}

#----------------------------------------------------------------------------------------
sub _feedPackagePickupScheduleObject {
    my ($PackagePickupSchedule) = @_;

    my $ShippingFulfillmentMethod = _createShippingFulfillmentMethod();
    $PackagePickupSchedule->setCredentials($ShippingFulfillmentMethod->get('WebserviceUsername'),
                                           $ShippingFulfillmentMethod->get('WebservicePassword'));

    my $OriginAddress      = _createAddress();
    my $ShippingLabel      = _createShippingLabel();

    $PackagePickupSchedule->setAddress($OriginAddress);
    $PackagePickupSchedule->setPickupSettings($ShippingLabel);
}

#----------------------------------------------------------------------------------------
sub _feedPackagePickupScheduleObjectWithWrongAddress {
    my ($PackagePickupSchedule) = @_;

    my $ShippingFulfillmentMethod = _createShippingFulfillmentMethod();
    $PackagePickupSchedule->setCredentials($ShippingFulfillmentMethod->get('WebserviceUsername'),
                                           $ShippingFulfillmentMethod->get('WebservicePassword'));

    my $OriginAddress      = _createWrongAddress();
    my $ShippingLabel      = _createShippingLabel();

    $PackagePickupSchedule->setAddress($OriginAddress);
    $PackagePickupSchedule->setPickupSettings($ShippingLabel);
}

#----------------------------------------------------------------------------------------
sub _createAddress {
    my $hAddressAttributes = {
        'AddressExtension' => 'RM 2100',
        'City'             => 'Washington',
        'Company'          => 'ePages',
        'FirstName'        => 'John',
        'LastName'         => 'Doe',
        'Phone'            => '123456789',
        'Fullname'         => 'John Doe',
        'State'            => 'DC',
        'Street'           => '475 Enfant Plaza SW',
        'Zipcode'          => '20260',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Address::API::Object::Address', $hAddressAttributes );
}

#----------------------------------------------------------------------------------------
sub _createWrongAddress {
    my $hAddressAttributes = {
        'AddressExtension' => 'RM 2100',
        'City'             => 'Washington',
        'Company'          => 'ePages',
        'FirstName'        => 'John',
        'LastName'         => 'Doe',
        'Phone'            => '123456789',
        'Fullname'         => 'John Doe',
        'State'            => 'DC',
        'Street'           => '',
        'Zipcode'          => '20260',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Address::API::Object::Address', $hAddressAttributes );
}

#----------------------------------------------------------------------------------------
sub _createShippingLabel {
    my $hLabelAttributes = {
        'ImageType'                  => 'None',
        'ParcelDepth'                => '5.30',
        'ParcelLength'               => '5.20',
        'ParcelWidth'                => '10.30',
        'ParcelGirth'                => '8.70',
        'PickupDate'                 => DateTime->today( time_zone => GetLocalTimeZone() ),
        'ShippingFulfillmentProduct' => _createShippingFulfillmentProduct(),
        'SizeUnit'                   => _createInchUnitOfMeasurement(),
        'TotalWeight'                => '10',
        'WeightUnit'                 => _createOunceUnitOfMeasurement(),
        'ServiceType'                => _createServiceType(),
        'PackageLocation'            => _createPackageLocation(),
        'SpecialInstructions'        => 'ring the bell',
    };

    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::USPS::API::Object::ShippingLabelUSPS',
        $hLabelAttributes );
}

#----------------------------------------------------------------------------------------
sub _createShippingFulfillmentProduct {
    my $hProductAttributes = { 'Alias' => 'PriorityMail', };
    return DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::ExternalShipping::API::Object::ShippingFulfillmentProduct',
        $hProductAttributes );
}
#----------------------------------------------------------------------------------------
sub _createServiceType {
    my $hServiceTypeAttributes = { 'Alias' => 'PRIORITY_MAIL', 'USPSWebservice2ndKey' => 'PriorityMail'};
    return DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::USPS::API::Object::ListItemUSPS',
        $hServiceTypeAttributes );
}
#----------------------------------------------------------------------------------------
sub _createPackageLocation {
    my $hPackageLocationAttributes = { 'Alias' => 'BACK_DOOR ', 'USPSWebserviceKey' => 'Back door'};
    return DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::USPS::API::Object::ListItemUSPS',
        $hPackageLocationAttributes );
}

#----------------------------------------------------------------------------------------
sub _createInchUnitOfMeasurement {
    my $hMetreAttributes = {
        'Alias' => 'metre',
        'Path'  => '/Units/metre',
    };
    my $metre = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Object::API::Object::UnitOfMeasurement',
        $hMetreAttributes );
    my $hInchAttributes = {
        'Alias'  => 'inch',
        'Factor' => 0.0254,
        'Parent' => $metre,
        'Path'   => '/Units/metre/inch',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Object::API::Object::UnitOfMeasurement',
        $hInchAttributes );
}

#----------------------------------------------------------------------------------------
sub _createOunceUnitOfMeasurement {
    my $hGramAttributes = { 'Alias' => 'gram' };
    my $Gram = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Object::API::Object::UnitOfMeasurement',
        $hGramAttributes );
    my $hOunceAttributes = {
        'Alias'  => 'ounce',
        'Factor' => 28.3495,
        'Parent' => $Gram,
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Object::API::Object::UnitOfMeasurement',
        $hOunceAttributes );
}


RunScript( Sub => \&Main );

1;
