#========================================================================================
# package      DE_EPAGES::USPS::t::ShippingOrder
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for ShippingOrder
#========================================================================================
package DE_EPAGES::USPS::t::ShippingOrder;
use strict;

use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Test::Mock::EpagesObject;
use DE_EPAGES::USPS::API::ShippingOrder;
use Test::More;
use Test::Exception;


sub Main {
    testIssueingOfMultipleProceededShippingLabels();
    testIssueingOf3ShippingLabelsWhereTheMiddleOneIsInvalid();
    testIssueingOfMultipleNoProceededShippingLabels();
    testNativDieErrorOnProcessing();
    done_testing();
}

#-----------------------------------------------------------------------------------------
sub testIssueingOfMultipleProceededShippingLabels {

    my $subTestName = ( caller (0) )[3];

    my $MockedShippingTransactionUSPS = _BuildMockedTransaction(2);

    my $ShippingOrder = _BuildMockedShippingOrder( [ 1, 1 ] );
    $ShippingOrder->trigger($MockedShippingTransactionUSPS);

    my $aTriggeredShippingLabels = $MockedShippingTransactionUSPS->get('ShippingLabels');
    my $ExpectedValueOfIssued    = 1;
    is( $MockedShippingTransactionUSPS->get('TransStatus'),
        'AllShippingLabelsProceeded', "$subTestName - transaction status - AllShippingLabelsProceeded" );
    is( $aTriggeredShippingLabels->[0]->get('Issued'),
        $ExpectedValueOfIssued, "$subTestName - label1 has expected value of 'issued' - 1" );
    is( $aTriggeredShippingLabels->[1]->get('Issued'),
        $ExpectedValueOfIssued, "$subTestName - label2 has expected value of 'issued' - 1" );

    return;
}

#-----------------------------------------------------------------------------------------
sub testIssueingOf3ShippingLabelsWhereTheMiddleOneIsInvalid {

    my $subTestName = ( caller (0) )[3];

    my $MockedShippingTransactionUSPS = _BuildMockedTransaction(3);

    my $ShippingOrder = _BuildMockedShippingOrder( [ 1, 0, 1 ] );
    $ShippingOrder->trigger($MockedShippingTransactionUSPS);

    my $aTriggeredShippingLabels = $MockedShippingTransactionUSPS->get('ShippingLabels');
    is( $MockedShippingTransactionUSPS->get('TransStatus'),
        'SomeShippingLabelsProceeded', "$subTestName - transaction status - SomeShippingLabelsProceeded" );
    is( $aTriggeredShippingLabels->[0]->get('Issued'), 1, "$subTestName - label1 has expected value of 'issued' - 1" );
    is( $aTriggeredShippingLabels->[1]->get('Issued'), 0, "$subTestName - label2 has expected value of 'issued' - 0" );
    is( $aTriggeredShippingLabels->[2]->get('Issued'), 1, "$subTestName - label3 has expected value of 'issued' - 1" );

    return;
}

#-----------------------------------------------------------------------------------------
sub testIssueingOfMultipleNoProceededShippingLabels {

    my $subTestName = ( caller (0) )[3];

    my $MockedShippingTransactionUSPS = _BuildMockedTransaction(2);

    my $ShippingOrder = _BuildMockedShippingOrder( [ 0, 0 ] );
    $ShippingOrder->trigger($MockedShippingTransactionUSPS);

    my $aTriggeredShippingLabels = $MockedShippingTransactionUSPS->get('ShippingLabels');
    my $ExpectedValueOfIssued    = 0;
    is( $MockedShippingTransactionUSPS->get('TransStatus'),
        'NoShippingLabelsProceeded', "$subTestName - transaction status - NoShippingLabelsProceeded" );
    is( $aTriggeredShippingLabels->[0]->get('Issued'),
        $ExpectedValueOfIssued, "$subTestName - label1 has expected value of 'issued' - 0" );
    is( $aTriggeredShippingLabels->[1]->get('Issued'),
        $ExpectedValueOfIssued, "$subTestName - label2 has expected value of 'issued' - 0" );

    return;
}

#-----------------------------------------------------------------------------------------
sub testNativDieErrorOnProcessing {

    my $subTestName = ( caller (0) )[3];

    my $MockedShippingTransactionUSPS = _BuildMockedTransaction(1);
    _SetCrashForFirstLabelOnTransaction($MockedShippingTransactionUSPS);

    my $ShippingOrder = _BuildMockedShippingOrder( [1] );

    throws_ok( sub { $ShippingOrder->trigger($MockedShippingTransactionUSPS); },
        qr/NativDie/, "$subTestName - dispatch that generates a NativDie" );
    return;
}

#-----------------------------------------------------------------------------------------
sub _BuildMockedTransaction {
    my ($numberOfLabels) = @_;

    my $hAttributes = {
        'NameOrAlias'    => 'TestTransactionAlias',
        'ShippingLabels' => _BuildMockedShippingLabels($numberOfLabels),
    };

    my $MockedShippingTransactionUSPS
      = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::USPS::API::Object::ShippingTransactionUSPS',
        $hAttributes );
    $MockedShippingTransactionUSPS->mock( 'issueShippingOrder', sub { return 1; } );

    return $MockedShippingTransactionUSPS;
}

#-----------------------------------------------------------------------------------------
sub _BuildMockedShippingLabels {
    my ($numberOfLabels) = @_;

    my $counter = 0;
    my @Labels  = ();
    while ( $counter < $numberOfLabels ) {
        my $Label = _BuildMockedShippingLabel();
        push ( @Labels, $Label );
        $counter++;
    }

    return \@Labels;
}

#-----------------------------------------------------------------------------------------
sub _BuildMockedShippingLabel {
    my ($hExtraAttributes) = @_;

    my $hAttributes = {
        'CrashProcess' => 0,
        'Issued'       => 0,
        'TrackingID'   => undef,
    };

    foreach my $Attribute ( keys ( %{$hExtraAttributes} ) ) {
        $hAttributes->{$Attribute} = $hExtraAttributes->{$Attribute};
    }
    my $MockedShippingLabelUSPS
      = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::USPS::API::Object::ShippingLabelUSPS', $hAttributes );
    $MockedShippingLabelUSPS->mock( 'generateTrackingLink', sub { return '/mocked/private/path/file'; } );

    return $MockedShippingLabelUSPS;
}

#-----------------------------------------------------------------------------------------
sub _SetCrashForFirstLabelOnTransaction {
    my ($MockedShippingTransactionUSPS) = @_;

    my $aShippingLabels = $MockedShippingTransactionUSPS->get('ShippingLabels');
    $aShippingLabels->[0]->set( { 'CrashProcess' => 1 } );

    return;
}

#-----------------------------------------------------------------------------------------
sub _BuildMockedShippingOrder {
    my ($aMockedResults) = @_;

    my $MockedShippingOrder = Test::MockObject::Extends->new('DE_EPAGES::USPS::API::ShippingOrder');

    $MockedShippingOrder->{'MockedResults'} = $aMockedResults;

    $MockedShippingOrder->mock(
        '_prepareShippingLabelWebService',
        sub {
            my $self = shift;
            my ($ShippingLabel) = @_;
            return Error('NativDie') if ( $ShippingLabel->get('CrashProcess') == 1 );
            return;
        }
    );

    $MockedShippingOrder->mock(
        '_generateShippingLabelOrError',
        sub {
            my $self               = shift;
            my $MockedResults      = $self->{'MockedResults'};
            my $IssuedSuccessfully = pop ( @{$MockedResults} );
            Error('TEST_ERROR') unless ($IssuedSuccessfully);
            return;
        }
    );
    $MockedShippingOrder->mock( '_saveShippingLabelFile', sub { return; } );
    $MockedShippingOrder->mock(
        '_reportAndUpdateShippingLabelOnSuccess',
        sub {
            my $self = shift;
            my ($ShippingLabel) = @_;
            $ShippingLabel->set( { 'Issued' => 1, 'TrackingID' => '222222' } );
            return;
        }
    );
    $MockedShippingOrder->mock(
        '_reportAndUpdateShippingLabelOnError',
        sub {
            my $self = shift;
            my ( $Error, $ShippingLabel ) = @_;
            $ShippingLabel->set( { 'Issued' => 0 } );
            return;
        }
    );

    return $MockedShippingOrder;
}

#=========================================================================================

RunScript( Sub => \&Main );

1;