#========================================================================================
# package      DE_EPAGES::USPS::t::TrackingDeliveryConfirmationLabel_external
# state        public
#----------------------------------------------------------------------------------------
# description  external integration tests for USPS Tracking-DeliveryConfirmationLabel API
#               (DeliveryConfirmationV4 USPS tag)
#========================================================================================
package DE_EPAGES::USPS::t::TrackingDeliveryConfirmationLabel_external;

use strict;

use DE_EPAGES::Core::API::Script qw ( RunScript );

use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw ( GetLocalTimeZone );
use DE_EPAGES::USPS::API::WebServices::TrackingDeliveryConfirmationLabel;
use DE_EPAGES::USPS::t::PermanenceTestingBuilder;

use DE_EPAGES::Test::Mock::EpagesObject;
use Test::More;


sub Main {

    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            if ( $ENV{SKIP_EXTERNAL_TESTS} ) {
                plan skip_all => 'call USPS Server (SKIP_EXTERNAL_TESTS == 1)';
            }

            eval {
                TestRequestValidationOnNoPersistence();
                TestRequestValidationOnPersistence();
            };
            if ( ExistsError() ) {
                my $Error = GetError();
                fail( 'Error: ' . $Error->longMessage );
            }
            done_testing();
        }
    );
}

#----------------------------------------------------------------------------------------
sub TestRequestValidationOnNoPersistence {
    my $SubTestName = ( caller (0) )[3];

    my $ShippingFulfillmentMethod = _CreateShippingFulfillmentMethod();
    my $TDCLabel = DE_EPAGES::USPS::API::WebServices::TrackingDeliveryConfirmationLabel->new($ShippingFulfillmentMethod);

    #now feed TrackingDeliveryConfirmationLabel object and check validity
    _FeedTrackingDeliveryConfirmationLabelObject($TDCLabel);
    ok( $TDCLabel->isValidRequestOrError(),
        "$SubTestName - called isValidRequestOrError method validator on a filled request" );

    $TDCLabel->callWebService();

    #lauch response and check response is valid
    ok( $TDCLabel->isValidResponseOrError(),           "$SubTestName - a valid response has been generated" );
    ok( $TDCLabel->getResponsePostageOrError() gt '0', "$SubTestName - postage value is greather than zero" );

}

#----------------------------------------------------------------------------------------
sub TestRequestValidationOnPersistence {
    my $SubTestName = ( caller (0) )[3];

    my $TestBuilderUSPS = DE_EPAGES::USPS::t::PermanenceTestingBuilder->new('TestShopPermanentDataForTestingUSPS.xml');

    SKIP:
    {
        ok($TestBuilderUSPS->isShopImported(), 'Test Shop for USPS testing imported into DataBase') or
            skip( 'Persistent Tests for USPS skipped' );

        my $ShippingFulfillmentMethod = $TestBuilderUSPS->getShippingFulfillmentMethod();
        my $ShippingLabel =
            $TestBuilderUSPS->getShopDependentObject('Customers/TestCustomer/Orders/5001/ShippingLabels/5001-1');

        my $TDCLabel =
            DE_EPAGES::USPS::API::WebServices::TrackingDeliveryConfirmationLabel->new($ShippingFulfillmentMethod);
        $TDCLabel->feedRequest($ShippingLabel);

        ok( $TDCLabel->callWebService(),
            "$SubTestName - called callWebService on a filled request for 5001-1 shipping label" );
        ok( $TDCLabel->isValidResponseOrError(), "$SubTestName - a valid response has been generated" );

        $TestBuilderUSPS->cleanupTestEnvironment();
    }

    return;
}

#----------------------------------------------------------------------------------------
sub _FeedTrackingDeliveryConfirmationLabelObject {
    my ($TDCLabel) = @_;

    my $ShippingLabel = _CreateShippingLabel();

    $TDCLabel->setDataFromShippingLabel($ShippingLabel);

    return;
}

#----------------------------------------------------------------------------------------
sub _CreateShippingLabel {
    my $hLabelAttributes = {
        'Address'                   => _CreateDestinationAddress(),
        'ContainerType'             => _CreateContainerType(),
        'ImageType'                 => 'None',
        'InformEndUserViaEmail'     => '1',
        'ParcelDepth'               => '5.30',
        'ParcelLength'              => '5.20',
        'ParcelWidth'               => '10.30',
        'ParcelGirth'               => '8.70',
        'PickupDate'                => DateTime->today( time_zone => GetLocalTimeZone() ),
        'ServiceType'               => _CreateServiceType(),
        'ShippingFulfillmentMethod' => _CreateShippingFulfillmentMethod(),
        'ShopAddress'               => _CreateOriginAddress(),
        'SizeUnit'                  => _CreateInchUnitOfMeasurement(),
        'TotalWeight'               => '10',
        'WeightUnit'                => _CreateOunceUnitOfMeasurement(),
    };

    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::USPS::API::Object::ShippingLabelUSPS',
        $hLabelAttributes );
}

#----------------------------------------------------------------------------------------
sub _CreateOriginAddress {
    my $hAddressAttributes = {
        'AddressExtension' => 'RM 2100',
        'City'             => 'Washington',
        'Company'          => 'ePages',
        'EMail'            => 'john.doe@gmail.com',
        'FullName'         => 'John Doe',
        'State'            => 'DC',
        'Street'           => '475 L\'Enfant Plaza SW',
        'Zipcode'          => '20260',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Address::API::Object::Address', $hAddressAttributes );
}

#----------------------------------------------------------------------------------------
sub _CreateDestinationAddress {
    my $hAddressAttributes = {
        'AddressExtension' => 'Ste 100',
        'City'             => 'Washington',
        'Company'          => 'XYZ Corporation',
        'FullName'         => 'Janice Dickens',
        'EMail'            => 'janice.dickens@gmail.com',
        'State'            => 'DC',
        'Street'           => '2 Massachusetts Ave NE',
        'Zipcode'          => '20212',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Address::API::Object::Address', $hAddressAttributes );
}

#----------------------------------------------------------------------------------------
sub _CreateServiceType {
    my $hProductAttributes = {
        'Alias'             => 'PRIORITY_MAIL',
        'USPSWebserviceKey' => 'Priority',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::USPS::API::Object::ListItemUSPS',
        $hProductAttributes );
}

#----------------------------------------------------------------------------------------
sub _CreateContainerType {
    my $hListItemAttributes = {
        'Alias'             => 'VARIABLE',
        'USPSWebserviceKey' => 'VARIABLE',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::USPS::API::Object::ListItemUSPS',
        $hListItemAttributes );
}

#----------------------------------------------------------------------------------------
sub _CreateInchUnitOfMeasurement {
    my $hMetreAttributes = {
        'Alias' => 'metre',
        'Path'  => '/Units/metre',
    };
    my $metre = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Object::API::Object::UnitOfMeasurement',
        $hMetreAttributes );
    my $hInchAttributes = {
        'Alias'  => 'inch',
        'Factor' => 0.0254,
        'Parent' => $metre,
        'Path'   => '/Units/metre/inch',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Object::API::Object::UnitOfMeasurement',
        $hInchAttributes );
}

#----------------------------------------------------------------------------------------
sub _CreateOunceUnitOfMeasurement {
    my $hGramAttributes = { 'Alias' => 'gram' };
    my $Gram = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Object::API::Object::UnitOfMeasurement',
        $hGramAttributes );
    my $hOunceAttributes = {
        'Alias'  => 'ounce',
        'Factor' => 28.3495,
        'Parent' => $Gram,
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Object::API::Object::UnitOfMeasurement',
        $hOunceAttributes );
}

#----------------------------------------------------------------------------------------
sub _CreateShippingFulfillmentMethod {
    my $hShippingFulfillmentMethodAttributes = {
        'Alias' => 'USPS',
        'WebservicePassword'  => '713BV14UD523',
        'WebserviceUsername'  => '605EPAGE1719',
        'Site' => _CreateMockShop(),
        'ShippingFulfillmentType' => _CreateShippingFulfillmentType(),
    };

    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::USPS::API::Object::ShippingFulfillmentMethodUSPS',
        $hShippingFulfillmentMethodAttributes );
}
#----------------------------------------------------------------------------------------
sub _CreateMockShop {
    my $hShopAttributes = { 'Alias' => 'Shop', 'IsInternalTestShop' => 1 };
    my $Shop = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::Shop::API::Object::Shop',
        $hShopAttributes );
    return $Shop;
}
#----------------------------------------------------------------------------------------
sub _CreateShippingFulfillmentType {
    my $hShippingFulfillmentTypeAttributes = { 'WebserviceURL' => 'https://stg-secure.shippingapis.com',
                                               'WebserviceTestURL' => 'https://stg-secure.shippingapis.com' };
    my $ShippingFulfillmentType = DE_EPAGES::Test::Mock::EpagesObject->new(
        'DE_EPAGES::ExternalShipping::API::Object::ShippingFulfillmentType',
        $hShippingFulfillmentTypeAttributes );
    return $ShippingFulfillmentType;
}


#=========================================================================================

RunScript( Sub => \&Main );

1;
