#========================================================================================
# package      DE_EPAGES::USPS::t::TrackingDeliveryConfirmationLabel_internal
# state        public
#----------------------------------------------------------------------------------------
# description  internal integration tests for USPS Tracking-DeliveryConfirmationLabel API
#               (DeliveryConfirmationV4 USPS tag)
#========================================================================================
package DE_EPAGES::USPS::t::TrackingDeliveryConfirmationLabel_internal;

use strict;

use DE_EPAGES::Core::API::Script qw ( RunScript );

use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw ( GetLocalTimeZone );
use DE_EPAGES::USPS::t::MockedCalls qw ( RemoteClientMocker );
use DE_EPAGES::USPS::API::WebServices::TrackingDeliveryConfirmationLabel;
use DE_EPAGES::USPS::t::PermanenceTestingBuilder;

use DE_EPAGES::Test::Mock::EpagesObject;
use Test::More;
use Test::Exception;
use File::Basename qw ( dirname );

use constant TEST_RESPONSES_DIRECTORY               => dirname(__FILE__) . '/TestResponses/';
use constant BIG_LENGTH                             => 10000;
use constant TEST_CENTIMETRES_VALUE_IN_INCHES       => 15.75;
use constant TEST_OUNCES_DECIMAL_VALUE              => 14.76;
use constant TEST_OUNCES_INTEGER_VALUE              => 15;
use constant TEST_KILOGRAMS_DECIMAL_VALUE           => 2.36;
use constant TEST_KILOGRAMS_TO_OUNCES_INTEGER_VALUE => 83;

sub Main {

    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            eval {
                TestRequestValidationOnNoPersistence();
                TestRequestValidationOnPersistence();
            };
            if ( ExistsError() ) {
                my $Error = GetError();
                fail( 'Error: ' . $Error->longMessage );
            }
            done_testing();
        }
    );
}

#----------------------------------------------------------------------------------------
sub TestRequestValidationOnNoPersistence {
    my $SubTestName = ( caller (0) )[3];

    # request validation tests
    my $ShippingFulfillmentMethod = _CreateShippingFulfillmentMethod();
    my $TDCLabelOnRequest
      = DE_EPAGES::USPS::API::WebServices::TrackingDeliveryConfirmationLabel->new($ShippingFulfillmentMethod);

    throws_ok(
        sub { $TDCLabelOnRequest->isValidRequestOrError() },
        qr/STRING_EMPTY/,
        "$SubTestName - called isValidRequestOrError method validator on a not filled request (mandatory strings empty"
    );

    _FeedTrackingDeliveryConfirmationLabelObject($TDCLabelOnRequest);
    ok( $TDCLabelOnRequest->isValidRequestOrError(),
        "$SubTestName - called isValidRequestOrError method validator on a filled request" );

    throws_ok(
        sub { $TDCLabelOnRequest->setRequestNodeEnumValueOrError( 'ImageType', 'InvalidImageTypeValue' ) },
        qr/INVALID_REQUEST_API_NODE_VALUE/,
        "$SubTestName - catched a try to set an invalid attribute value"
    );

    # auxiliary conditions tests
    my $ShippingLabelOnAuxConditions = _CreateShippingLabel();
    my $TDCLabelOnAuxConditions
      = DE_EPAGES::USPS::API::WebServices::TrackingDeliveryConfirmationLabel->new($ShippingFulfillmentMethod);
    $TDCLabelOnAuxConditions->setDataFromShippingLabel($ShippingLabelOnAuxConditions);

    my $oldDate = DateTime->new( 'year' => 2013, 'month' => 12, 'day' => 22 );
    $TDCLabelOnAuxConditions->setLabelDate($oldDate);
    throws_ok(
        sub { $TDCLabelOnAuxConditions->isValidRequestOrError() },
        qr/INVALID_REQUEST_NO_CONDITIONS_MET/,
        "$SubTestName - called isValidRequestOrError method validator on an invalid date condition"
    );
    $TDCLabelOnAuxConditions->setLabelDate( DateTime->today( time_zone => GetLocalTimeZone() ) );

    $TDCLabelOnAuxConditions->setRequestNodeBooleanValueOrError( 'AddressServiceRequested', 'true' );
    throws_ok(
        sub { $TDCLabelOnAuxConditions->isValidRequestOrError() },
        qr/INVALID_REQUEST_NO_CONDITIONS_MET/,
        "$SubTestName - called isValidRequestOrError method validator on an address service - image parameter failing condition"
    );

    # Size dimensions and containers dependencies tests
    my $ShippingLabelOnSize = _CreateShippingLabel();
    my $TDCLabelOnSize
      = DE_EPAGES::USPS::API::WebServices::TrackingDeliveryConfirmationLabel->new($ShippingFulfillmentMethod);
    $TDCLabelOnSize->setDataFromShippingLabel($ShippingLabelOnSize);

    is( $TDCLabelOnSize->getRequest()->getTextContentByXMLnodeName('Size'),
        'REGULAR', "$SubTestName - package size is set originally as 'REGULAR'" );

    $ShippingLabelOnSize->set( { 'ParcelLength' => 15.35 } );
    $TDCLabelOnSize->setDataFromShippingLabel($ShippingLabelOnSize);

    is( $TDCLabelOnSize->getRequest()->getTextContentByXMLnodeName('Size'),
        'LARGE',
        "$SubTestName - package size is set now changed as 'LARGE' because of the introduced package dimensions" );

    $TDCLabelOnSize->setSizeInInches( { 'ParcelGirth' => 40 }, _CreateCentimetreUnitOfMeasurement() );
    is(
        $TDCLabelOnSize->getRequest()->getTextContentByXMLnodeName('Girth'),
        TEST_CENTIMETRES_VALUE_IN_INCHES,
        "$SubTestName - Girth parcel dimension set into inches value from centimetres"
    );

    throws_ok( sub { $TDCLabelOnSize->setSizeInInches( { 'ParcelGirth' => 5 }, _CreateGallonUnitOfMeasurement() ) },
        qr/NO_LENGTH_UNIT/, "$SubTestName - catched a try to set gallons instead of inches" );

    # Weight dependencies tests
    my $ShippingLabelOnWeight = _CreateShippingLabel();
    $ShippingLabelOnWeight->set( { 'TotalWeight' => TEST_OUNCES_DECIMAL_VALUE } );
    my $TDCLabelOnWeight
      = DE_EPAGES::USPS::API::WebServices::TrackingDeliveryConfirmationLabel->new($ShippingFulfillmentMethod);
    $TDCLabelOnWeight->setDataFromShippingLabel($ShippingLabelOnWeight);

    is( $TDCLabelOnWeight->getRequest()->getTextContentByXMLnodeName('WeightInOunces'),
        TEST_OUNCES_INTEGER_VALUE, "$SubTestName - Weight set as an integer" );

    $TDCLabelOnWeight->setWeightInOuncesOrError( TEST_KILOGRAMS_DECIMAL_VALUE, _CreateKilogramUnitOfMeasurement() );
    is(
        $TDCLabelOnWeight->getRequest()->getTextContentByXMLnodeName('WeightInOunces'),
        TEST_KILOGRAMS_TO_OUNCES_INTEGER_VALUE,
        "$SubTestName - Weight converted from Kg into Ounces and set as an integer"
    );

    #positive response tests
    my $Client
      = RemoteClientMocker( TEST_RESPONSES_DIRECTORY . 'TrackingDeliveryConfirmationLabel_positive_response.xml',
        '200' );
    my $TDCLabelWithPositiveResponse
      = DE_EPAGES::USPS::API::WebServices::TrackingDeliveryConfirmationLabel->new( $ShippingFulfillmentMethod,
        $Client );
    _FeedTrackingDeliveryConfirmationLabelObject($TDCLabelWithPositiveResponse);
    $TDCLabelWithPositiveResponse->callWebService();

    ok(
        $TDCLabelWithPositiveResponse->isValidResponseOrError(),
        "$SubTestName - called isValidResponseOrError on a valid response"
    );

    is(
        $TDCLabelWithPositiveResponse->getResponseDeliveryConfirmationNumberOrError(),
        '420202129405501699320000409479',
        "$SubTestName - called getResponseDeliveryConfirmationNumberOrError"
    );

    is( $TDCLabelWithPositiveResponse->getResponsePostageOrError(),
        '5.60', "$SubTestName - called getResponsePostageOrError" );

    cmp_ok( length ( $TDCLabelWithPositiveResponse->getResponseLabelBase64CodeOrError() ),
        '>', BIG_LENGTH,
        "$SubTestName - called getResponseLabelBase64CodeOrError and got a label longer than 200 characters" );

    like( $TDCLabelWithPositiveResponse->getResponseLogMessageOrEmpty(),
        qr/^Warning/, "$SubTestName - called getResponseLogMessageOrEmpty and got a Warning message" );

    #error response tests
    $Client
      = RemoteClientMocker( TEST_RESPONSES_DIRECTORY . 'TrackingDeliveryConfirmationLabel_error_response.xml', '200' );
    my $TDCLabelWithErrorResponse
      = DE_EPAGES::USPS::API::WebServices::TrackingDeliveryConfirmationLabel->new( $ShippingFulfillmentMethod,
        $Client );
    _FeedTrackingDeliveryConfirmationLabelObject($TDCLabelWithErrorResponse);

    $TDCLabelWithErrorResponse->callWebService();
    ok( $TDCLabelWithErrorResponse->isErrorResponse(),
        "$SubTestName - called isErrorResponse method validator on an erroneus response" );

    like(
        $TDCLabelWithErrorResponse->getErrorResponseDescriptionOrEmpty(),
        qr/^Authorization failure/,
        "$SubTestName - called getResponseErrorDescriptionOrEmpty and got an Authorization failure message"
    );

    is( $TDCLabelWithErrorResponse->getErrorResponseNumberOrEmpty(),
        '80040b1a', "$SubTestName - called getErrorResponseNumberOrEmpty" );

    is( $TDCLabelWithErrorResponse->getErrorResponseSourceOrEmpty(),
        'UspsCom::DoAuth', "$SubTestName - called getErrorResponseSourceOrEmpty" );

    is( $TDCLabelWithErrorResponse->getErrorResponseHelpFileOrEmpty(),
        '', "$SubTestName - called getErrorResponseHelpFileOrEmpty" );

    is( $TDCLabelWithErrorResponse->getErrorResponseHelpContextOrEmpty(),
        '', "$SubTestName - called getErrorHelpContextOrEmpty" );

    $Client = RemoteClientMocker( TEST_RESPONSES_DIRECTORY . 'TestResponses/Failed_Connection.txt', '500' );
    my $TDCLabelWithErrorConnection
      = DE_EPAGES::USPS::API::WebServices::TrackingDeliveryConfirmationLabel->new( $ShippingFulfillmentMethod,
        $Client );
    _FeedTrackingDeliveryConfirmationLabelObject($TDCLabelWithErrorConnection);
    throws_ok( sub { $TDCLabelWithErrorConnection->callWebService() },
        qr/CONNECTION_FAILED/, "$SubTestName - catched a Connection Failed error" );

    return;
}

#----------------------------------------------------------------------------------------
sub TestRequestValidationOnPersistence {
    my $SubTestName = ( caller (0) )[3];

    my $TestBuilderUSPS    = DE_EPAGES::USPS::t::PermanenceTestingBuilder->new('TestShopPermanentDataForTestingUSPS.xml');

  SKIP:
    {
        ok( $TestBuilderUSPS->isShopImported(), 'Test Shop for USPS testing imported into DataBase' ) or
            skip( 'Persistent Tests for USPS skipped' );

        my $ShippingFulfillmentMethod = $TestBuilderUSPS->getShippingFulfillmentMethod();
        my $Client                    = RemoteClientMocker(
            TEST_RESPONSES_DIRECTORY . 'TrackingDeliveryConfirmationLabel_positive_response.xml',
            '200'
        );

        my $TDCLabel1
          = DE_EPAGES::USPS::API::WebServices::TrackingDeliveryConfirmationLabel->new( $ShippingFulfillmentMethod,
            $Client );
        my $ShippingLabel1
          = $TestBuilderUSPS->getShopDependentObject('Customers/TestCustomer/Orders/5001/ShippingLabels/5001-1');
        $TDCLabel1->feedRequest($ShippingLabel1);

        my $TDCLabel2
          = DE_EPAGES::USPS::API::WebServices::TrackingDeliveryConfirmationLabel->new( $ShippingFulfillmentMethod,
            $Client );
        my $ShippingLabel2
          = $TestBuilderUSPS->getShopDependentObject('Customers/TestCustomer/Orders/5001/ShippingLabels/5001-2');
        $TDCLabel2->feedRequest($ShippingLabel2);

        is( $TDCLabel1->getRequest()->getTextContentByXMLnodeName('Size'),
            'LARGE', "$SubTestName - package size is set as 'LARGE' for 5001-1 shipping label" );
        is( $TDCLabel2->getRequest()->getTextContentByXMLnodeName('Size'),
            'REGULAR', "$SubTestName - package size is set as 'REGULAR' for 5001-2 shipping label" );

        ok( $TDCLabel1->isValidRequestOrError(),
            "$SubTestName - called isValidRequestOrError method validator on a filled request sourced from DB (5001-1 shipping label)"
        );
        ok( $TDCLabel1->callWebService(),
            "$SubTestName - called callWebService method request on a filled request sourced from DB (5001-1 shipping label)"
        );
        ok( $TDCLabel2->isValidRequestOrError(),
            "$SubTestName - called isValidRequestOrError method validator on a filled request sourced from DB (5001-2 shipping label)"
        );
        ok( $TDCLabel2->callWebService(),
            "$SubTestName - called callWebService method request on a filled request sourced from DB (5001-2 shipping label)"
        );

        $TestBuilderUSPS->cleanupTestEnvironment();
    }

    return;
}

#----------------------------------------------------------------------------------------
sub _FeedTrackingDeliveryConfirmationLabelObject {
    my ($TDCLabel) = @_;

    my $ShippingLabel = _CreateShippingLabel();
    $TDCLabel->feedRequest($ShippingLabel);

    return;
}

#----------------------------------------------------------------------------------------
sub _CreateShippingLabel {
    my $hLabelAttributes = {
        'Address'                   => _CreateDestinationAddress(),
        'ContainerType'             => _CreateContainerType(),
        'ImageType'                 => 'None',
        'InformEndUserViaEmail'     => '1',
        'Order'                     => _CreateOrder(),
        'ParcelDepth'               => '5.30',
        'ParcelLength'              => '5.20',
        'ParcelWidth'               => '10.30',
        'ParcelGirth'               => '8.70',
        'PickupDate'                => DateTime->today( time_zone => GetLocalTimeZone() ),
        'ServiceType'               => _CreateServiceType(),
        'ShippingFulfillmentMethod' => _CreateShippingFulfillmentMethod(),
        'ShopAddress'               => _CreateOriginAddress(),
        'SizeUnit'                  => _CreateInchUnitOfMeasurement(),
        'TotalWeight'               => '10',
        'WeightUnit'                => _CreateOunceUnitOfMeasurement(),
    };

    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::USPS::API::Object::ShippingLabelUSPS',
        $hLabelAttributes );
}

#----------------------------------------------------------------------------------------
sub _CreateOrder {
    my $hOrderAttributes = { 'LineItemContainer' => _CreateContainer(), };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Order::API::Object::CustomerOrder',
        $hOrderAttributes );
}

#----------------------------------------------------------------------------------------
sub _CreateContainer {
    my $hContainerAttributes = { 'Shipping' => _CreateLineItemShipping(), };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Order::API::Object::LineItemContainer',
        $hContainerAttributes );
}

#----------------------------------------------------------------------------------------
sub _CreateLineItemShipping {
    my $hLineItemShippingAttributes = { 'IsUSPSSuggestedAddress' => '1', };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Order::API::Object::LineItemShipping',
        $hLineItemShippingAttributes );
}

#----------------------------------------------------------------------------------------
sub _CreateOriginAddress {
    my $hAddressAttributes = {
        'AddressExtension' => 'RM 2100',
        'City'             => 'Washington',
        'Company'          => 'ePages',
        'EMail'            => 'john.doe@gmail.com',
        'FullName'         => 'John Doe',
        'State'            => 'DC',
        'Street'           => '475 L\'Enfant Plaza SW',
        'Zipcode'          => '20260',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Address::API::Object::Address', $hAddressAttributes );
}

#----------------------------------------------------------------------------------------
sub _CreateDestinationAddress {
    my $hAddressAttributes = {
        'AddressExtension' => 'Ste 100',
        'City'             => 'Washington',
        'Company'          => 'XYZ Corporation',
        'EMail'            => 'janice.dickens@gmail.com',
        'FullName'         => 'Janice Dickens',
        'State'            => 'DC',
        'Street'           => '2 Massachusetts Ave NE',
        'Zipcode'          => '20212',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Address::API::Object::Address', $hAddressAttributes );
}

#----------------------------------------------------------------------------------------
sub _CreateServiceType {
    my $hListItemAttributes = {
        'Alias'             => 'PRIORITY_MAIL',
        'USPSWebserviceKey' => 'Priority',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::USPS::API::Object::ListItemUSPS',
        $hListItemAttributes );
}

#----------------------------------------------------------------------------------------
sub _CreateContainerType {
    my $hListItemAttributes = {
        'Alias'             => 'RECTANGULAR',
        'USPSWebserviceKey' => 'RECTANGULAR',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::USPS::API::Object::ListItemUSPS',
        $hListItemAttributes );
}

#----------------------------------------------------------------------------------------
sub _CreateMetreUnitOfMeasurement {
    my $hMetreAttributes = {
        'Alias' => 'metre',
        'Path'  => '/Units/metre',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Object::API::Object::UnitOfMeasurement',
        $hMetreAttributes );
}

#----------------------------------------------------------------------------------------
sub _CreateInchUnitOfMeasurement {
    my $hInchAttributes = {
        'Alias'  => 'inch',
        'Factor' => 0.0254,
        'Parent' => _CreateMetreUnitOfMeasurement(),
        'Path'   => '/Units/metre/inch',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Object::API::Object::UnitOfMeasurement',
        $hInchAttributes );
}

#----------------------------------------------------------------------------------------
sub _CreateCentimetreUnitOfMeasurement {
    my $hCentimetreAttributes = {
        'Alias'  => 'centimetre',
        'Factor' => 0.01,
        'Parent' => _CreateMetreUnitOfMeasurement(),
        'Path'   => '/Units/metre/centimetre',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Object::API::Object::UnitOfMeasurement',
        $hCentimetreAttributes );
}

#----------------------------------------------------------------------------------------
sub _CreateGramUnitOfMeasurement {
    my $hGramAttributes = {
        'Alias' => 'gram',
        'Path'  => '/Units/gram',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Object::API::Object::UnitOfMeasurement',
        $hGramAttributes );
}

#----------------------------------------------------------------------------------------
sub _CreateOunceUnitOfMeasurement {
    my $hOunceAttributes = {
        'Alias'  => 'ounce',
        'Factor' => 28.3495,
        'Parent' => _CreateGramUnitOfMeasurement(),
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Object::API::Object::UnitOfMeasurement',
        $hOunceAttributes );
}

#----------------------------------------------------------------------------------------
sub _CreateKilogramUnitOfMeasurement {
    my $hKilogramAttributes = {
        'Alias'  => 'kilogram',
        'Factor' => 1000,
        'Parent' => _CreateGramUnitOfMeasurement(),
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Object::API::Object::UnitOfMeasurement',
        $hKilogramAttributes );
}

#----------------------------------------------------------------------------------------
sub _CreateGallonUnitOfMeasurement {
    my $hLitreAttributes = {
        'Alias' => 'litre',
        'Path'  => '/Units/litre',
    };
    my $litre = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Object::API::Object::UnitOfMeasurement',
        $hLitreAttributes );
    my $hGallonAttributes = {
        'Alias'  => 'gallon',
        'Factor' => 3.78541,
        'Parent' => $litre,
        'Path'   => '/Units/litre/gallon',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Object::API::Object::UnitOfMeasurement',
        $hGallonAttributes );
}

#----------------------------------------------------------------------------------------
sub _CreateShippingFulfillmentMethod {
    my $hShippingFulfillmentMethodAttributes = {
        'Alias'                   => 'USPS',
        'WebservicePassword'      => 'AnyPassword',
        'WebserviceUsername'      => 'AnyUsername',
        'Site'                    => _CreateMockShop(),
        'ShippingFulfillmentType' => _CreateShippingFulfillmentType(),
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::USPS::API::Object::ShippingFulfillmentMethodUSPS',
        $hShippingFulfillmentMethodAttributes );
}

#----------------------------------------------------------------------------------------
sub _CreateMockShop {
    my $hShopAttributes = { 'Alias' => 'Shop', 'IsInternalTestShop' => 1 };
    my $Shop = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::Shop::API::Object::Shop', $hShopAttributes );
    return $Shop;
}

#----------------------------------------------------------------------------------------
sub _CreateShippingFulfillmentType {
    my $hShippingFulfillmentTypeAttributes = {
        'WebserviceURL'     => 'https://stg-secure.shippingapis.com',
        'WebserviceTestURL' => 'https://stg-secure.shippingapis.com'
    };
    my $ShippingFulfillmentType
      = DE_EPAGES::Test::Mock::EpagesObject->new( 'DE_EPAGES::ExternalShipping::API::Object::ShippingFulfillmentType',
        $hShippingFulfillmentTypeAttributes );
    return $ShippingFulfillmentType;
}

#=========================================================================================

RunScript( Sub => \&Main );

1;
